// Local includes
#include "backendservice.h"
#include "backendservice_adaptor.h"
#include "pythonproviderplugin.h"

// Util includes
#include <providerplugin.hpp>
#include <pluginxmlfile.h>

// Global includess
#include <QPluginLoader>
#include <QDir>
#include <QDomDocument>
#include <QCoreApplication>
#include <QDebug>

class BackendServicePrivate : public QObject
{
    Q_OBJECT

public:
    // Properties

    // key=providerId
    QHash<QString, ProviderPlugin*> plugins;

    // Functions
    void unloadPlugins();
    void loadPlugins();
    bool loadPlugin(const QString& absolutePath);

protected:
    ProviderPlugin* loadPluginLibrary(const QString& absolutePath) const;
    ProviderPlugin* loadNativePluginLibrary(const QString& absolutePath) const;
    ProviderPlugin* loadPythonPluginLibrary(const QString& absolutePath) const;
};

void BackendServicePrivate::unloadPlugins()
{
    // Sanity check
    if ( plugins.isEmpty() )
        return;

    qDebug() << "Unloading" << plugins.size() << "plugins";

    // Cleanup memory
    qDeleteAll(plugins);
    plugins.clear();
}

void BackendServicePrivate::loadPlugins()
{
    // Define the plugin directory
    //QDir pluginDir("/opt/web2sms/plugins");
    QDir pluginDir("/home/gri/plugins");

    // Iterate the plugins directory
    foreach(const QString& subdir, pluginDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot)) {
        pluginDir.cd(subdir);

        QStringList nameFilters;
        nameFilters << "*.xml";

        // List the xml files and load the plugins
        foreach(const QString& xmlfile, pluginDir.entryList(nameFilters, QDir::Files)) {
            // Try to load the plugin
            loadPlugin(pluginDir.absoluteFilePath(xmlfile));
        }

        pluginDir.cdUp();
    }
}

bool BackendServicePrivate::loadPlugin(const QString &absolutePath)
{
    PluginXmlFile file;

    // Read the xml file
    if ( file.loadFromFile(absolutePath) == false ) {
        qDebug() << "Failed to read xml file" << absolutePath << ":" << file.errorString();
        return false;
    }

    // Load the plugin
    ProviderPlugin* plugin = loadPluginLibrary(file.pluginPath());
    if ( plugin == NULL )
        return false;

    // Store the plugin information
    plugins.insert(file.providerId(), plugin);

    return true;
}


ProviderPlugin* BackendServicePrivate::loadPluginLibrary(const QString& absolutePath) const
{
    // Get information about the file
    QFileInfo fileInfo(absolutePath);

    // Check what plugin type we have
    QString suffix = fileInfo.suffix();

    // Is it python?
    if ( suffix == "py" )
        return loadPythonPluginLibrary(absolutePath);
    else if ( QLibrary::isLibrary(absolutePath) ) // Native plugin?
        return loadNativePluginLibrary(absolutePath);
    else {
        qDebug() << "Unable to load unknown plugin file:" << absolutePath;
        return NULL;
    }
}

ProviderPlugin* BackendServicePrivate::loadNativePluginLibrary(const QString& absolutePath) const
{
    // Create the plugin loader
    QPluginLoader loader(absolutePath);
    // TODO: Check if we need to set the load hints
    loader.setLoadHints(QLibrary::ExportExternalSymbolsHint);

    // Load the plugin
    ProviderPlugin* plugin = qobject_cast<ProviderPlugin*>( loader.instance() );

    // Did it load?
    if ( plugin == NULL )
        qDebug() << "Failed to load plugin" << absolutePath << ":" << loader.errorString();

    return plugin;
}

ProviderPlugin* BackendServicePrivate::loadPythonPluginLibrary(const QString& absolutePath) const
{
    PythonProviderPlugin* plugin = new PythonProviderPlugin;
    if ( plugin->loadScript(absolutePath) == false ) {
        qDebug() << "Failed to load python plugin" << absolutePath << ":" << plugin->errorString();

        // Reset plugin
        delete plugin;
        plugin = NULL;
    }

    return plugin;
}

BackendService::BackendService(QObject *parent)
: QObject(parent)
, d(new BackendServicePrivate)
{
    // Register our D-Bus adaptor
    new Web2smsbackendAdaptor(this);
}

BackendService::~BackendService()
{
    delete d;
}

void BackendService::reloadPlugins()
{
    // Unload the plugins
    d->unloadPlugins();
    d->loadPlugins();
}

bool BackendService::reloadPlugin(const QString &absolutePath)
{
    qDebug() << "Reloading plugin" << absolutePath;

    Q_UNUSED(absolutePath);
    return false;
}

QStringList BackendService::providers() const
{
    return d->plugins.keys();
}

void BackendService::stop()
{
    // Quit the application
    QCoreApplication::quit();
}

QString createErrorMessage(const QString& msg)
{
    return QString("{\"error\":{\"message\":\"%1\"}}").arg(msg);
}

QString BackendService::sendMessage(const QString &providerId,
                                    const QString &login,
                                    const QString &password,
                                    const QVariantList &numbers,
                                    const QString &smsText)
{
    // Get the right plugin
    ProviderPlugin* plugin = d->plugins.value(providerId);
    if ( plugin == NULL ) {
        return createErrorMessage( QString("provider '%1' does not exist").arg(providerId) );
    }

    // Create a provider interface
    ProviderInterface* iface = plugin->createProvider(providerId.toUtf8());
    if ( iface == NULL ) {
        return createErrorMessage( QString("provider '%1' exists but can't create the provider interface").arg(providerId) );
    }

    ContactInfoList receivers;
    foreach(const QVariant& number, numbers) {
        ContactInfo info;
        info.setNumber(number.toString());

        receivers.append(info);
    }

    iface->sendMessage(login, password, receivers, smsText, QVariantMap());

    return QString();
}

#include "backendservice.moc"
