// Local includes
#include "pythonproviderinterface.h"

// Global includes
#include <QMapIterator>
#include <QDebug>

struct PythonProviderInterfacePrivate
{
    PythonProviderInterfacePrivate()
    : provider(NULL)
    {
    }

    // Properties
    PyObject* provider;
    QString errorString;

    // Functions
    PyObject* callFunction(PyObject* object, const QByteArray& functionName, PyObject* args);
    void cleanup();

    PyObject* createTupleFromMessage(const QString &Login, const QString &Password, const ContactInfoList &receivers, const QString &SmsText, const QVariantMap &PluginDict) const;
    PyObject* pyStringFromQString(const QString& str) const;
};

PyObject* PythonProviderInterfacePrivate::callFunction(PyObject *object, const QByteArray &functionName, PyObject *args)
{
    // Clear the error string
    errorString.clear();

    // Get the handle to the "plugin_instance" function
    PyObject* instanceFunction = PyObject_GetAttrString(object, functionName);
    if ( instanceFunction == NULL ) {
        errorString = QString("%1 not found").arg(QString::fromLatin1(functionName));
        return NULL;
    }

    // Make sure it's a callable function
    if ( !PyCallable_Check(instanceFunction) ) {
        Py_DecRef(instanceFunction);
        errorString = QString("%1 is not a callable function").arg(QString::fromLatin1(functionName));
        return NULL;
    }

    // Call the function
    PyObject* provider = PyObject_CallObject(instanceFunction, args);
    Py_DecRef(instanceFunction);

    return provider;
}

void PythonProviderInterfacePrivate::cleanup()
{
    if ( provider == NULL )
        return;

    Py_DecRef(provider);
    provider = NULL;
}

PyObject* PythonProviderInterfacePrivate::createTupleFromMessage(const QString& Login, const QString& Password, const ContactInfoList& receivers, const QString& SmsText, const QVariantMap& PluginDict) const
{
    // TODO: We have to get the login data
    PyObject* login = pyStringFromQString(Login);
    PyObject* password = pyStringFromQString(Password);
    PyObject* smstext = pyStringFromQString(SmsText);

    PyObject* numbers = PyList_New(0);
    foreach(const ContactInfo& contactInfo, receivers) {
        // Create the numbers dictionaries
        PyObject* name = pyStringFromQString(contactInfo.name());
        PyObject* number = pyStringFromQString(contactInfo.number());

        PyObject* dict = PyDict_New();
        if ( contactInfo.name().isEmpty() == false )
            PyDict_SetItemString(dict, "name", name);
        PyDict_SetItemString(dict, "number", number);

        // Append to the list of numbers
        PyList_Append(numbers, dict);

        // Memory cleanups
        Py_DecRef(name);
        Py_DecRef(number);
        Py_DecRef(dict);
    }

    PyObject* pluginDict = PyDict_New();
    {
        QMapIterator<QString, QVariant> iter(PluginDict);
        while ( iter.hasNext() ) {
            QByteArray key = iter.key().toUtf8();
            PyObject* value = pyStringFromQString(iter.value().toString());
            PyDict_SetItemString(pluginDict, key, value);
            Py_DecRef(value);

            iter.next();
        }

        // TODO: Remove the debugging fail value
        PyObject* fail = PyBool_FromLong(0);
        PyDict_SetItemString(pluginDict, "FAIL", fail);
        Py_DecRef(fail);
    }

    // def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT)
    PyObject* value = Py_BuildValue("OOOOO", login, password, numbers, smstext, pluginDict);

    // Memory cleanups
    Py_DecRef(login);
    Py_DecRef(password);
    Py_DecRef(numbers);
    Py_DecRef(smstext);
    Py_DecRef(pluginDict);

    return value;
}

PyObject* PythonProviderInterfacePrivate::pyStringFromQString(const QString &str) const
{
    QByteArray data = str.toUtf8();
    return PyString_FromString(data.constData());
}

PythonProviderInterface::PythonProviderInterface(QObject *parent)
: ProviderInterface(parent)
, d(new PythonProviderInterfacePrivate)
{
}

PythonProviderInterface::~PythonProviderInterface()
{
    d->cleanup();
    delete d;
}

bool PythonProviderInterface::initializeFromScript(PyObject *script)
{
    // Try to cleanup memory first
    d->cleanup();

    // Sanity check
    if ( script == NULL )
        return false;

    PyObject* tuple = PyTuple_New(0);
    PyObject* provider = d->callFunction(script, "plugin_instance", tuple);
    Py_DecRef(tuple);

    // Stop here if we already have an error
    if ( d->errorString.isEmpty() == false )
        return false;

    // Just check if we received an object
    if ( provider == NULL ) {
        d->errorString = "plugin_instance did not return any object";
        return false;
    }

    // Remember that object
    d->provider = provider;
    return true;
}

QString PythonProviderInterface::errorString() const
{
    return d->errorString;
}

bool PythonProviderInterface::sendMessage(const QString &login, const QString &password, const ContactInfoList &numbers, const QString &smsText, const QVariantMap &pluginDict)
{
    // Sanity check
    if ( d->provider == NULL )
        return false;

    PyObject* tuple = d->createTupleFromMessage(login, password, numbers, smsText, pluginDict);
    PyObject* msg = d->callFunction(d->provider, "sendSMS", tuple);
    Py_DecRef(tuple);

    if ( PyErr_Occurred() )
        PyErr_Print();

    // Stop here if we already have an error
    if ( d->errorString.isEmpty() == false )
        return false;

    // Just check if we received an object
    if ( msg == NULL ) {
        d->errorString = "sendSMS did not return any object";
        return false;
    }

    // Convert object to a string
    PyObject* strObject = PyObject_Str(msg);

    // Get the returned string
    char* str = PyString_AsString(strObject);
    QByteArray bt(str);

    qDebug() << "sendSMS returned" << bt;

    Py_DecRef(strObject);
    Py_DecRef(msg);

    return true;
}
