// Local includes
#include "pythonproviderplugin.h"
#include "pythonproviderinterface.h"

// Global includes
#include <QDir>
#include <QFileInfo>
#include <shiboken/shiboken.h>
#include <pyside_qtcore_python.h>
#include <stdlib.h>

struct PythonProviderPluginPrivate
{
    PythonProviderPluginPrivate()
    : script(NULL)
    {
    }

    // Properties
    PyObject* script;
    QString errorString;

    // Functions
    void adjustSystemPath(const QString& absolutePath);
};

void PythonProviderPluginPrivate::adjustSystemPath(const QString &absolutePath)
{
    // Get the directory
    QFileInfo fileInfo(absolutePath);
    QDir dir = fileInfo.dir();

    QByteArray currentPath = dir.absolutePath().toUtf8();

    // Adjust the python system path
    PyObject* sysPath = PySys_GetObject("path");
    if ( sysPath ) {
        PyObject* path = PyString_FromString(currentPath.constData());
        PyList_Append(sysPath, path);
        Py_DecRef(path);
    }
}

PythonProviderPlugin::PythonProviderPlugin(QObject *parent)
: QObject(parent)
, d(new PythonProviderPluginPrivate)
{
    // Initialize python if we haven't done this now
    if ( !Py_IsInitialized() ) {
        Py_Initialize();

        // Register a cleanup handler
        //atexit( Py_Finalize() );
    }
}

PythonProviderPlugin::~PythonProviderPlugin()
{
    unloadScript();

    delete d;
}

bool PythonProviderPlugin::loadScript(const QString &absolutePath)
{
    // Adjust the system path
    d->adjustSystemPath(absolutePath);

    QFileInfo fileInfo(absolutePath);
    QByteArray baseName = fileInfo.baseName().toUtf8();

    // Load the python script
    PyObject* fileName = PyString_FromString(baseName.constData());
    d->script = PyImport_Import(fileName);
    Py_DecRef(fileName);

    PyErr_Print();

    return !PyErr_Occurred();
}

void PythonProviderPlugin::unloadScript()
{
    // Decrease the reference count
    if ( d->script )
        Py_DecRef(d->script);
}

QString PythonProviderPlugin::errorString() const
{
    return d->errorString;
}

ProviderInterface* PythonProviderPlugin::createProvider(const QByteArray &providerId) const
{
    PythonProviderInterface* iface = new PythonProviderInterface;
    if ( iface->initializeFromScript(d->script) == false ) {
        d->errorString = iface->errorString();
        delete iface;
        return NULL;
    }

    return iface;
}
