// Local includes
#include "accountsmodel.h"

// Global includes
#include <accounts-qt/Accounts/manager.h>
#include <QDebug>

class AccountsModelPrivate : public QObject
{
    Q_OBJECT

public:
    AccountsModelPrivate()
    {
        // Connections
        connect(&manager, SIGNAL(accountCreated(Accounts::AccountId)), this, SLOT(onAccountCreated(Accounts::AccountId)));
        connect(&manager, SIGNAL(accountRemoved(Accounts::AccountId)), this, SLOT(onAccountRemoved(Accounts::AccountId)));
    }

    AccountsModel* self;

    // Properties
    Accounts::Manager manager;
    QList<Accounts::Account*> accounts;

    QString providerFilter;

    // Functions
    void init();
    bool matchesFilter(Accounts::Account* account) const;

public slots:
    void onAccountCreated(Accounts::AccountId id);
    void onAccountRemoved(Accounts::AccountId id);
    void onAccountDisplayNameChanged();
};

void AccountsModelPrivate::init()
{
    // Initial state
    foreach(const Accounts::AccountId& accountId, manager.accountList()) {
        // Get the current account
        Accounts::Account* account = manager.account(accountId);

        // Skip this account?
        if ( !matchesFilter(account) )
            continue;

        // Connect to name changes
        connect(account, SIGNAL(displayNameChanged(QString)), this, SLOT(onAccountDisplayNameChanged()));

        // Add to our list
        accounts.append(account);
    }

    emit self->countChanged();
}

bool AccountsModelPrivate::matchesFilter(Accounts::Account *account) const
{
    return providerFilter.isEmpty() || account->providerName() == providerFilter;
}

void AccountsModelPrivate::onAccountCreated(Accounts::AccountId id)
{
    // ### Should we check if the account already exists?

    // Append the account at the bottom of the list
    Accounts::Account* account = manager.account(id);
    if ( account == NULL ) {
        qDebug() << "Invalid account supplied";
        return;
    }

    // Ignore?
    if ( !matchesFilter(account) )
        return;

    // Connect to name changes
    connect(account, SIGNAL(displayNameChanged(QString)), this, SLOT(onAccountDisplayNameChanged()));

    // Append the account
    int pos = accounts.size();
    self->beginInsertRows(QModelIndex(), pos, pos);
    accounts.append(account);
    self->endInsertRows();

    emit self->countChanged();
}

void AccountsModelPrivate::onAccountRemoved(Accounts::AccountId id)
{
    // Find the removed account and remove it from the list
    for(int i=0; i < accounts.size(); ++i) {
        // Ge the current account
        const Accounts::Account* account = accounts.at(i);

        // Does the id match?
        if ( account->id() != id )
            continue;

        // Disconnect name changes
        disconnect(account, SIGNAL(displayNameChanged(QString)), this, SLOT(onAccountDisplayNameChanged()));

        // Remove the account
        self->beginRemoveRows(QModelIndex(), i, i);
        accounts.removeAt(i);
        self->endRemoveRows();

        emit self->countChanged();

        break;
    }
}

void AccountsModelPrivate::onAccountDisplayNameChanged()
{
    // Get the account
    Accounts::Account* account = qobject_cast<Accounts::Account*>(sender());

    // Get the row
    int row = accounts.indexOf(account);
    if ( row == -1 )
        return;

    QModelIndex index = self->index(row, 0);
    emit self->dataChanged(index, index);
}

AccountsModel::AccountsModel(QObject *parent)
: QAbstractListModel(parent)
, d(new AccountsModelPrivate)
{
    d->self = this;

    connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SIGNAL(providerFilterChanged()));
    connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SIGNAL(providerFilterChanged()));

    QHash<int, QByteArray> roleNames;
    roleNames.insert(DisplayNameRole, "displayName");
    roleNames.insert(IdRole, "id");
    roleNames.insert(EnabledRole, "enabled");
    roleNames.insert(ProviderIdRole, "providerId");
    roleNames.insert(UsernameRole, "username");
    roleNames.insert(AccountRole, "account");

    // For support in ListDelegate
    roleNames.insertMulti(DisplayNameRole, "title");
    //roleNames.insertMulti(EnabledRole, "subtitle");

    setRoleNames(roleNames);
}

AccountsModel::~AccountsModel()
{
    delete d;
}

void AccountsModel::setProviderFilter(const QString &providerFilter)
{
    if ( d->providerFilter == providerFilter )
        return;

    d->providerFilter = providerFilter;
    emit providerFilterChanged();
}

QString AccountsModel::providerFilter() const
{
    return d->providerFilter;
}

int AccountsModel::rowCount(const QModelIndex &parent) const
{
    if ( parent.isValid() )
        return 0;

    return d->accounts.size();
}

QVariant AccountsModel::data(const QModelIndex &index, int role) const
{
    // Sanity check
    if ( index.row() < 0 || index.row() >= d->accounts.size() )
        return QVariant();

    // Get the selected account
    const Accounts::Account* account = d->accounts.at(index.row());

    switch(role) {
    case DisplayNameRole:
        return account->displayName();

    case IdRole:
        return QString::number(account->id());

    case EnabledRole:
        if ( account->enabled() )
            //% "Enabled"
            return qtTrId("qtn_web2sms_accountsmodel_enabled");
        else
            //% "Disabled"
            return qtTrId("qtn_web2sms_accountsmodel_disabled");

    case ProviderIdRole:
        return account->valueAsString("providerId");

    case UsernameRole:
        return account->valueAsString("username");

    case AccountRole:
        return QVariant(); // QVariant::fromValue(account);

    default:
        break;
    }

    return QVariant();
}

int AccountsModel::count() const
{
    return rowCount();
}

void AccountsModel::classBegin()
{
}

void AccountsModel::componentComplete()
{
    // Initialize the model
    d->init();
}

int AccountsModel::indexFromAccountId(quint32 accountId) const
{
    // Find the removed account and remove it from the list
    for(int i=0; i < d->accounts.size(); ++i) {
        // Ge the current account
        const Accounts::Account* account = d->accounts.at(i);

        // Does the id match?
        if ( account->id() != accountId )
            continue;

        return i;
    }

    return -1;
}

Accounts::Account * AccountsModel::account(int row) const
{
    return d->accounts.at(row);
}

#include "accountsmodel_meego.moc"
