// Local includes
#include "accountwrapper.h"

// Global includes
#include <accounts-qt/account.h>
#include <signon-qt/SignOn/identity.h>
#include <signon-qt/SignOn/identityinfo.h>
#include <signon-qt/SignOn/authsession.h>
#include <qjson/serializer.h>
#include <qjson/parser.h>
#include <QHash>
#include <QDebug>

class AccountWrapperPrivate : public QObject
{
    Q_OBJECT

public:
    AccountWrapperPrivate()
    : account(NULL)
    , status(AccountWrapper::Unknown)
    , identity(NULL)
    , enabled(false)
    {
    }

    AccountWrapper* self;

    // Properties
    Accounts::Account* account;

    AccountWrapper::Status status;

    SignOn::Identity* identity;
    SignOn::IdentityInfo identityInfo;
    SignOn::AuthSessionP session;

    QString displayName;
    bool enabled;
    QString username;
    QString password;

    QHash<QString, QVariant> valueCache;

    // Functions
    void storeIdentity();

public slots:
    void didSync();
    void didStoreCredentials(const quint32 id);
    void didNotStoreCredentials(const SignOn::Error& error);

    void sessionDidRespond(const SignOn::SessionData& sessionData);
    void sessionDidNotRespond(const SignOn::Error& error);
};

void AccountWrapperPrivate::storeIdentity()
{
    if ( identity == NULL ) {
        // Create a new identity if needed
        identity = SignOn::Identity::newIdentity();

        // Save that information
        connect(identity, SIGNAL(credentialsStored(quint32)), this, SLOT(didStoreCredentials(quint32)));
        connect(identity, SIGNAL(error(SignOn::Error)), this, SLOT(didNotStoreCredentials(SignOn::Error)));
    }

    // Store our valuable information in the identity
    identityInfo.setCaption(self->providerId());
    identityInfo.setUserName(username);
    identityInfo.setSecret(password, true);
    //identityInfo.setOwner("AID::.web2sms.");
    //identityInfo.setAccessControlList( QStringList() << "AID::.web2sms." );
    identityInfo.setMethod("password", QStringList() << "password");

    // Save it!
    identity->storeCredentials(identityInfo);
}

void AccountWrapperPrivate::didSync()
{
}

void AccountWrapperPrivate::didStoreCredentials(const quint32 id)
{
    qDebug() << Q_FUNC_INFO << id;

    // Update the credentials id of the account
    account->setCredentialsId(id);
    account->sync();
}

void AccountWrapperPrivate::didNotStoreCredentials(const SignOn::Error &error)
{
    qDebug() << Q_FUNC_INFO << error.message();
}

void AccountWrapperPrivate::sessionDidRespond(const SignOn::SessionData& sessionData)
{
    qDebug() << Q_FUNC_INFO << sessionData.UserName() << sessionData.Secret().size();

    username = sessionData.UserName();
    password = sessionData.Secret();
    status = AccountWrapper::Ready;

    emit self->usernameChanged();
    emit self->passwordChanged();
    emit self->statusChanged();
}

void AccountWrapperPrivate::sessionDidNotRespond(const SignOn::Error& error)
{
    qDebug() << Q_FUNC_INFO << error.message();

    status = AccountWrapper::Error;
    emit self->statusChanged();
}

AccountWrapper::AccountWrapper(QObject *parent)
: QObject(parent)
, d(new AccountWrapperPrivate)
{
    d->self = this;
}

AccountWrapper::~AccountWrapper()
{
    if ( d->identity && d->session ) {
        d->identity->destroySession(d->session);
        d->session = NULL;
    }

    delete d->identity;
    delete d;
}

AccountWrapper::Status AccountWrapper::status() const
{
    return d->status;
}

void AccountWrapper::setAccount(Accounts::Account *account)
{
    // Sanity check
    if ( d->account == account )
        return;

    // Disconnect old connections
    if ( d->account ) {
        disconnect(d->account, SIGNAL(displayNameChanged(QString)), this, SIGNAL(displayNameChanged()));
        disconnect(d->account, SIGNAL(synced()), this, SLOT(updateCachedEntries()));
        disconnect(d->account, SIGNAL(synced()), this, SIGNAL(synced()));

        if ( d->identity && d->session ) {
            d->identity->destroySession(d->session);
            d->session = NULL;
        }

        delete d->identity;
        d->identity = NULL;
    }

    // Remember the account
    d->account = account;
    d->status = Unknown;

    // Make new connections
    if ( d->account ) {
        connect(d->account, SIGNAL(displayNameChanged(QString)), this, SIGNAL(displayNameChanged()));
        connect(d->account, SIGNAL(synced()), this, SLOT(updateCachedEntries()));
        connect(d->account, SIGNAL(synced()), this, SIGNAL(synced()));
    }

    updateCachedEntries();
}

Accounts::Account * AccountWrapper::account() const
{
    return d->account;
}

void AccountWrapper::setDisplayName(const QString &displayName)
{
    if ( d->displayName == displayName )
        return;

    if ( d->account )
        d->account->setDisplayName(displayName);

    d->displayName = displayName;
    emit displayNameChanged();
}

QString AccountWrapper::displayName() const
{
    return d->displayName;
}

void AccountWrapper::setEnabled(bool enabled)
{
    if ( d->enabled == enabled )
        return;

    if ( d->account )
        d->account->setEnabled(enabled);

    d->enabled = enabled;
    emit enabledChanged();
}

bool AccountWrapper::enabled() const
{
    return d->enabled;
}

void AccountWrapper::setUsername(const QString& username)
{
    if ( d->username == username )
        return;

    d->username = username;
    emit usernameChanged();
}

QString AccountWrapper::username() const
{
    return d->username;
}

void AccountWrapper::setPassword(const QString& password)
{
    if ( d->password == password )
        return;

    d->password = password;
    emit passwordChanged();
}

QString AccountWrapper::password() const
{
    return d->password;
}

void AccountWrapper::setProviderId(const QString &providerId)
{
    setValue("providerId", providerId);
    emit providerIdChanged();
}

QString AccountWrapper::providerId() const
{
    return valueAsString("providerId");
}

void AccountWrapper::setPluginCredentials(const QVariant& pluginCredentials)
{
    // Save JSon data
    QJson::Serializer serializer;

    QString data = serializer.serialize(pluginCredentials);
    qDebug() << __PRETTY_FUNCTION__ << ":" << data;

    setValue("pluginCredentials", data);
}

QVariant AccountWrapper::pluginCredentials() const
{
    QString data = valueAsString("pluginCredentials");

    qDebug() << __PRETTY_FUNCTION__ << ":" << data;

    // TODO: Check whether this succeeded or not
    QJson::Parser parser;
    return parser.parse(data.toUtf8());
}

void AccountWrapper::setBalance(const QString &balance)
{
    if ( this->balance() == balance )
        return;

    setValue("balance", balance);
    emit balanceChanged();
}

QString AccountWrapper::balance() const
{
    return valueAsString("balance");
}

void AccountWrapper::setValue(const QString &key, const QVariant &value)
{
    d->valueCache.insert(key, value);
    d->account->setValue(key, value);
}

QString AccountWrapper::valueAsString(const QString &key) const
{
    // Looks strange, but does its job
    return d->valueCache.value(key, d->account->valueAsString(key)).toString();
}

void AccountWrapper::remove()
{
    // Sanity check
    if ( d->account == NULL )
        return;

    // Remove the identity from password storage
    if ( d->identity == NULL )
        d->identity = SignOn::Identity::existingIdentity( d->account->credentialsId() );

    if ( d->identity ) {
        d->identity->remove();
        d->identity->deleteLater();
        d->identity = NULL;

        qDebug() << Q_FUNC_INFO << "removed identity";
    }
    else
        qDebug() << Q_FUNC_INFO << "did not remove identity";

    // Remove the account from account storage
     d->account->remove();
}

void AccountWrapper::sync()
{
    // Sanity check
    if ( d->account == NULL )
        return;

    // Manually set the username. Otherwise all models would need to read the identity
    d->account->setValue("username", d->username);

    qDebug() << Q_FUNC_INFO;

    // We store the identity, that will call account->sync()
    d->storeIdentity();
}

void AccountWrapper::updateCachedEntries()
{
    bool statusUpdated = false;

    d->displayName = d->account->displayName();
    d->enabled = d->account->enabled();

    if ( d->account->credentialsId() ) {
        // Get the identity
        if ( d->identity == NULL ) {
            d->identity = SignOn::Identity::existingIdentity(d->account->credentialsId());

            connect(d->identity, SIGNAL(credentialsStored(quint32)), d, SLOT(didStoreCredentials(quint32)));
            connect(d->identity, SIGNAL(error(SignOn::Error)), d, SLOT(didNotStoreCredentials(SignOn::Error)));

            qDebug() << Q_FUNC_INFO << "loaded identity" << d->account->credentialsId();
        }

        // Query our known information
        if ( d->identity ) {
            // Get session data
            if ( d->session == NULL ) {
                // Create a new session if needed
                d->session = d->identity->createSession("password");
                if ( d->session ) {
                    connect(d->session, SIGNAL(response(SignOn::SessionData)), d, SLOT(sessionDidRespond(SignOn::SessionData)));
                    connect(d->session, SIGNAL(error(SignOn::Error)), d, SLOT(sessionDidNotRespond(SignOn::Error)));

                    // Prepare session data
                    SignOn::SessionData info;
                    info.setUiPolicy( SignOn::NoUserInteractionPolicy );

                    // Update the status
                    statusUpdated = true;
                    d->status = Loading;
                    emit statusChanged();

                    d->session->process(info, "password");
                } else {
                    qDebug() << "Unable to start authsession for password method";
                }
            }
        }
    }

    emit displayNameChanged();
    emit enabledChanged();

    if ( statusUpdated == false ) {
        d->status = Ready;
        emit statusChanged();
    }
}

#include "accountwrapper_meego.moc"
