// Local includes
#include "backend.h"
#include "accountwrapper.h"

// Global includes
#ifdef MEEGO_EDITION_HARMATTAN

namespace Accounts {
    class Provider;
    class AgProvider;
    class Service;
    class AgService;
    typedef QList<Service*> ServiceList;
}

#include <accounts-qt/Accounts/Service>
#include <accounts-qt/Accounts/Manager>
#include <AccountSetup/ProviderPluginProcess>
#endif // MEEGO_EDITION_HARMATTAN
#include <QApplication>
#include <QThread>
#include <QWidget>
#include <QDebug>

class BackendPrivate
{
public:
    BackendPrivate()
    : accountWrapper(NULL)
#ifdef MEEGO_EDITION_HARMATTAN
    , manager(NULL)
#endif // MEEGO_EDITION_HARMATTAN
    , messageHandler(NULL)
    , debugMode(false)
    {
    }

    ~BackendPrivate()
    {
        delete accountWrapper;
#ifdef MEEGO_EDITION_HARMATTAN
        delete manager;
#endif // MEEGO_EDITION_HARMATTAN
    }

    AccountWrapper* accountWrapper;

#ifdef MEEGO_EDITION_HARMATTAN
    Accounts::Manager* manager;
#endif // MEEGO_EDITION_HARMATTAN
    MessageHandler* messageHandler;
    bool debugMode;
};

Backend::Backend(QObject *parent)
: QObject(parent)
, d(new BackendPrivate)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Get the instance of the provider plugin
    AccountSetup::ProviderPluginProcess* plugin = AccountSetup::ProviderPluginProcess::instance();

    // Update the account
    if ( plugin && plugin->account() ) {
        d->accountWrapper = new AccountWrapper;
        d->accountWrapper->setAccount( plugin->account() );
    }
#endif // MEEGO_EDITION_HARMATTAN
}

Backend::~Backend()
{
    delete d;
}

void Backend::setDebugMode(bool debugMode)
{
    if ( d->debugMode == debugMode )
        return;

    d->debugMode = debugMode;
    emit debugModeChanged();
}

bool Backend::debugMode() const
{
    return d->debugMode;
}

void Backend::raiseWindow()
{
    foreach(QWidget* widget, QApplication::topLevelWidgets()) {
        // Ignore everything except QDeclarativeView
        if ( widget->inherits("QDeclarativeView") == false )
            continue;

        widget->showFullScreen();
        qDebug() << "Raised widget" << widget;
    }
}

AccountWrapper* Backend::createAccount(const QString& providerId)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Delete existing accounts
    if ( d->accountWrapper )
        d->accountWrapper->deleteLater();

    // Create a new manager if needed
    if ( d->manager == NULL ) {
        d->manager = new Accounts::Manager;
    }

    // Create a new account and enable it by default
    Accounts::Account* account = d->manager->createAccount("web2sms");

    d->accountWrapper = new AccountWrapper;
    d->accountWrapper->setAccount(account);
    d->accountWrapper->setEnabled(true);
    d->accountWrapper->setProviderId(providerId);

    // Notify the change
    emit accountChanged();

#endif // MEEGO_EDITION_HARMATTAN

    return d->accountWrapper;
}

AccountWrapper * Backend::accountFromId(const QString &accountId)
{
#ifdef MEEGO_EDITION_HARMATTAN
    qDebug() << Q_FUNC_INFO << accountId << QThread::currentThreadId();

    // Delete existing accounts
    if ( d->accountWrapper ) {
        d->accountWrapper->deleteLater();
        d->accountWrapper = NULL;
    }

    if ( d->manager == NULL ) {
        d->manager = new Accounts::Manager;
    }

    if ( accountId.toInt() == 0 ) {
        emit accountChanged();
        return NULL;
    }

    // Load an existing account
    Accounts::Account* account = d->manager->account( accountId.toInt() );
    if ( account == NULL ) {
        emit accountChanged();
        return NULL;
    }

    d->accountWrapper = new AccountWrapper;
    d->accountWrapper->setAccount(account);

    // Notify the change
    emit accountChanged();
#endif // MEEGO_EDITION_HARMATTAN

    return d->accountWrapper;
}

AccountWrapper * Backend::account() const
{
    return d->accountWrapper;
}

PluginResolver * Backend::plugins() const
{
    return PluginResolver::instance();
}

MessageHandler * Backend::messageHandler() const
{
    if ( d->messageHandler == NULL )
        d->messageHandler = new MessageHandler( const_cast<Backend*>(this) );

    return d->messageHandler;
}

bool Backend::isInternationalNumber(const QString &number) const
{
    return number.startsWith("+") || number.startsWith("00");
}
