// Local includes
#include "contactmodel.h"
#include "contactpicker.h"

// Global includes
#include <QContactManager>
#include <QContactFetchRequest>
#include <QContactDisplayLabel>
#include <QContactName>
#include <QContactPhoneNumber>
#include <QContactUnionFilter>
#include <QContactDetailFilter>
#include <QDebug>

QTM_USE_NAMESPACE

class ContactModelPrivate : public QObject
{
    Q_OBJECT

public:
    ContactModelPrivate()
#ifdef MEEGO_EDITION_HARMATTAN
    : manager("tracker")
#endif
    {
        autoUpdate = true;
    }

    ContactModel* self;

    // Properties
    QContactManager manager;

    bool autoUpdate;
    QString nameFilter;
    QVariantList contacts;

    // Functions
    QList<QContactSortOrder> prepareSortOrder() const;
    QContactFilter prepareFilter() const;
    void getContacts();

public slots:
    void didFetchContacts();
};

QList<QContactSortOrder> ContactModelPrivate::prepareSortOrder() const
{
    QList<QContactSortOrder> order;
    ContactHelper helper;

    // Setup ordering
    QContactSortOrder orderDisplayName;
    orderDisplayName.setDetailDefinitionName(QContactDisplayLabel::DefinitionName, QContactDisplayLabel::FieldLabel);
    orderDisplayName.setCaseSensitivity(Qt::CaseInsensitive);
    orderDisplayName.setDirection(Qt::AscendingOrder);

    QLatin1String fieldName = QContactName::FieldLastName;
    if ( helper.nameOrder() == ContactHelper::FirstLast )
        fieldName = QContactName::FieldFirstName;

    QContactSortOrder orderName;
    orderName.setDetailDefinitionName(QContactName::DefinitionName, fieldName);
    orderName.setCaseSensitivity(Qt::CaseInsensitive);
    orderName.setDirection(Qt::AscendingOrder);

    order.append(orderDisplayName);
    order.append(orderName);
    return order;
}

QContactFilter ContactModelPrivate::prepareFilter() const
{
    // If there is no filter set, we don't filter
    if ( nameFilter.isEmpty() )
        return QContactFilter();

    // Split the name into segments
    QContactUnionFilter unionFilter;
    foreach(const QString& segment, nameFilter.split(' ', QString::SkipEmptyParts)) {
        QContactDetailFilter firstNameFilter;
        firstNameFilter.setDetailDefinitionName(QContactName::DefinitionName, QContactName::FieldFirstName);
        firstNameFilter.setMatchFlags(QContactFilter::MatchStartsWith);
        firstNameFilter.setValue(segment);
        unionFilter.append(firstNameFilter);

        QContactDetailFilter lastNameFilter;
        lastNameFilter.setDetailDefinitionName(QContactName::DefinitionName, QContactName::FieldLastName);
        lastNameFilter.setMatchFlags(QContactFilter::MatchStartsWith);
        lastNameFilter.setValue(segment);
        unionFilter.append(lastNameFilter);
    }

    return unionFilter;
}

void ContactModelPrivate::getContacts()
{
    QContactFetchRequest* request = new QContactFetchRequest;
    connect(request, SIGNAL(resultsAvailable()), this, SLOT(didFetchContacts()));
    request->setManager(&manager);
    request->setSorting( prepareSortOrder() );
    request->setFilter( prepareFilter() );
    request->start();
}

void ContactModelPrivate::didFetchContacts()
{
    self->beginResetModel();
    contacts.clear();

    ContactHelper helper;

    // Get the sending request
    QContactFetchRequest* request = qobject_cast<QContactFetchRequest*>( sender() );

    // Iterate the found contacts
    foreach(const QContact& contact, request->contacts()) {
        // Ignore contacts without phone numbers
        QList<QContactPhoneNumber> numbers = contact.details<QContactPhoneNumber>();
        if ( numbers.isEmpty() )
            continue;

        // Convert the phone numbers into a variant list
        QVariantList phoneNumbers;
        foreach(const QContactPhoneNumber& number, numbers) {
            QVariantMap phoneNumber;
            phoneNumber.insert("number", number.number());

            phoneNumbers.append(phoneNumber);
        }

        // Find a useful name for the contact
        QContactName fieldName = contact.detail<QContactName>();
        QString displayLabel = helper.nameOrder() == ContactHelper::FirstLast ? fieldName.firstName() : fieldName.lastName();
        if ( displayLabel.isEmpty() )
            displayLabel = contact.displayLabel();

        // Prepare the contact
        QVariantMap c;
        c.insert("section", displayLabel.at(0));
        c.insert("phoneNumbers", phoneNumbers);
        c.insert("displayLabel", contact.displayLabel());
        c.insert("contactId", contact.localId()); // TODO: Local id or id?
        contacts.append(c);
    }

    request->deleteLater();
    self->endResetModel();
}

ContactModel::ContactModel(QObject *parent)
: QAbstractListModel(parent)
, d(new ContactModelPrivate)
{
    d->self = this;

    // Prepare the role names
    QHash<int, QByteArray> roleNames;
    roleNames.insert(ContactRole, "contact");
    roleNames.insert(DisplayLabelRole, "displayLabel");
    roleNames.insert(SectionRole, "section");

    setRoleNames(roleNames);
}

ContactModel::~ContactModel()
{
    delete d;
}

void ContactModel::setAutoUpdate(bool autoUpdate)
{
    if ( d->autoUpdate == autoUpdate )
        return;

    d->autoUpdate = autoUpdate;
    emit autoUpdateChanged();
}

bool ContactModel::autoUpdate() const
{
    return d->autoUpdate;
}

void ContactModel::setNameFilter(const QString &nameFilter)
{
    if ( d->nameFilter == nameFilter )
        return;

    d->nameFilter = nameFilter;
    emit nameFilterChanged();

    if ( d->autoUpdate )
        d->getContacts();
}

QString ContactModel::nameFilter() const
{
    return d->nameFilter;
}

QVariantList ContactModel::contacts() const
{
    return d->contacts;
}

void ContactModel::update()
{
    d->getContacts();
}

void ContactModel::classBegin()
{
}

void ContactModel::componentComplete()
{
    // Fetch the contacts
    if ( d->autoUpdate )
        d->getContacts();
}

int ContactModel::rowCount(const QModelIndex &parent) const
{
    if ( parent.isValid() )
        return 0;

    return d->contacts.size();
}

QVariant ContactModel::data(const QModelIndex &index, int role) const
{
    // Get the contact
    const QVariantMap& contact = d->contacts.at(index.row()).toMap();

    switch(role)
    {
    case ContactRole:
        return contact;

    case DisplayLabelRole:
        return contact.value("displayLabel");

    case SectionRole:
        return contact.value("section");

    default:
        break;
    }

    return QVariant();
}

#include "contactmodel.moc"
