#include "conversationmodel.h"

#include <QDBusConnection>
#include <QThread>

using CommHistory::Event;
using ContextProvider::Property;

ConversationModel::ConversationModel(QObject *parent)
: QAbstractListModel(parent)
, m_status(Unknown)
, m_initialized(false)
, m_groupId(-1)
, m_service(QDBusConnection::SessionBus, "net.muhkuh.web2sms.Context")
{
    m_observedConversation = new Property(m_service, "Messaging.ObservedConversation", this);

    connect(&m_model, SIGNAL(modelReady(bool)), this, SLOT(onModelReady(bool)));
    connect(&m_model, SIGNAL(modelReset()), this, SLOT(onModelReset()));
    connect(&m_model, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(onRowsInserted(QModelIndex,int,int)));
    connect(&m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(onRowsRemoved(QModelIndex,int,int)));
    connect(&m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SLOT(onDataChanged(QModelIndex,QModelIndex)));

    QThread* thread = new QThread(this);
    thread->start();

    m_model.setBackgroundThread(thread);
    m_model.setFirstChunkSize(6);
    m_model.setChunkSize(6);

    QHash<int, QByteArray> roleNames;
    roleNames.insert(TextRole, "text");
    roleNames.insert(StatusRole, "status");
    roleNames.insert(StatusTextRole, "statusText");
    roleNames.insert(DirectionRole, "direction");
    roleNames.insert(EndTimeRole, "endTime");
    roleNames.insert(MessageTokenRole, "messageToken");
    roleNames.insert(EventIdRole, "eventId");
    roleNames.insert(IsReadRole, "isRead");

    setRoleNames(roleNames);
}

ConversationModel::~ConversationModel()
{
    QThread* thread = m_model.backgroundThread();
    thread->terminate();
    thread->wait();
}

void ConversationModel::setGroupId(int groupId)
{
    if ( m_groupId == groupId )
        return;

    m_groupId = groupId;
    emit groupIdChanged();

    if ( m_initialized ) {
        setStatus(Loading);

        m_model.getEvents(m_groupId);
    }
}

int ConversationModel::groupId() const
{
    return m_groupId;
}

void ConversationModel::setObservedConversation(const QString &observedConversation)
{
    if ( observedConversation.isEmpty() )
        m_observedConversation->unsetValue();
    else {
        QString localId = "/org/freedesktop/Telepathy/Account/ring/tel/ring";
        QString remoteId = observedConversation;

        QVariantList value;
        value << QVariant::fromValue(localId) // LocalId
              << QVariant::fromValue(remoteId) // RemoteId
              << QVariant::fromValue(0); // ChatType::ChatType_P2P

        m_observedConversation->setValue( QVariant::fromValue(value) );

        // Mark all messages in this conversation as read
        markReadMessages();
    }

    emit observedConversationChanged();
}

QString ConversationModel::observedConversation() const
{
    return QString();
}

void ConversationModel::setStatus(ConversationModel::Status status)
{
    if ( m_status == status )
        return;

    m_status = status;
    emit statusChanged();
}

void ConversationModel::markReadMessages()
{
    QList<CommHistory::Event> unreadEvents;

    // Mark unread messages and decide if we should play a sound for the incoming one
    foreach(CommHistory::Event event, m_events) {
        if ( event.isRead() )
            continue;

        event.setIsRead(true);
        unreadEvents << event;
    }

    if ( unreadEvents.isEmpty() == false ) {
        m_model.modifyEvents(unreadEvents);
        qDebug() << "Marked" << unreadEvents.size() << "messages as read.";
    }
}

ConversationModel::Status ConversationModel::status() const
{
    return m_status;
}

bool ConversationModel::deleteEvent(int eventId)
{
    return m_model.deleteEvent(eventId);
}

void ConversationModel::classBegin()
{
}

void ConversationModel::componentComplete()
{
    m_initialized = true;

    if ( m_groupId != -1 ) {
        // Adjust the status
        setStatus(Loading);

        // Start the loading
        m_model.getEvents(m_groupId);
    }
}

QVariant ConversationModel::data(const QModelIndex &index, int role) const
{
    // Get the event
    Event event = m_events.at(index.row());

    switch(role) {
    case TextRole:
        return QVariant::fromValue(event.freeText());

    case StatusRole:
        return event.status();

    case StatusTextRole:
        {
            switch(event.status()) {
            case Event::SendingStatus:
                //% "Sending"
                return qtTrId("qtn_web2sms_status_sending");

            case Event::SentStatus:
                //% "Sent"
                return qtTrId("qtn_web2sms_status_sent");

            case Event::DeliveredStatus:
                //% "Delivered"
                return qtTrId("qtn_web2sms_status_delivered");

            case Event::TemporarilyFailedStatus:
                //% "Temporarily failed"
                return qtTrId("qtn_web2sms_status_temporarily_failed");

            case Event::PermanentlyFailedStatus:
                //% "Failed"
                return qtTrId("qtn_web2sms_status_permanently_failed");

            case Event::UnknownStatus:
            default:
                //% "Unknown"
                return qtTrId("qtn_web2sms_status_unknown");
            }
        }

    case DirectionRole:
        return (event.direction() == CommHistory::Event::Inbound) ? "Inbound" : "Outbound";

    case EndTimeRole:
        {
            QDateTime now = QDateTime::currentDateTime();
            QDateTime dt = event.endTime().toLocalTime();

            QString time = dt.time().toString(Qt::SystemLocaleShortDate);
            if ( dt.date() == now.date() ) {
                return time;
            } else {
                QString date = dt.date().toString(Qt::SystemLocaleShortDate);
                //% "%1 | %2"
                //: "DateTime format in GroupModel. %1 is the Date, %2 is the Time. Normal format is date | time if not the same day"
                return qtTrId("qtn_web2sms_datetime_format").arg(date).arg(time);
            }
        }

    case MessageTokenRole:
        return event.messageToken();

    case EventIdRole:
        return event.id();

    case IsReadRole:
        return QVariant::fromValue( event.isRead() );

    default:
        return QVariant();
    }
}

bool ConversationModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    qDebug() << Q_FUNC_INFO << index << value << roleNames().value(role);
    return false;
}

void ConversationModel::onModelReady(bool success)
{
    setStatus( success ? Ready : Error );
}

void ConversationModel::onModelReset()
{
    m_events.clear();
    reset();
}

int ConversationModel::rowCount(const QModelIndex &) const
{
    return m_events.size();
}

void ConversationModel::onRowsInserted(const QModelIndex &, int start, int end)
{
    bool playSound = (!m_events.isEmpty() && (start==0 || end==0));

    QList<CommHistory::Event> unreadEvents;

    beginInsertRows(QModelIndex(), start, end);

    bool incoming = false;
    for(int i=start; i <= end; ++i) {
        QModelIndex index = m_model.index(i, 0);
        CommHistory::Event event = m_model.event(index);
        m_events.insert(i, event);

        // Remember unread messages
        if ( event.isRead() == false )
            unreadEvents << event;

        // Check that atleast one message is incoming
        if ( event.direction() == CommHistory::Event::Inbound )
            incoming = true;
    }

    endInsertRows();

    // Change the setting for observed conversations
    if ( m_observedConversation->isSet() )
        markReadMessages();

    // Trigger the incoming message signal when we have to play a sound
    if ( incoming && playSound ) {
        emit incomingMessage();
    }
}

void ConversationModel::onRowsRemoved(const QModelIndex &, int source_start, int source_end)
{
    // Map the values to our view. Remember the values are inverted
    int start = source_end; //mapFromSource(source_end);
    int end = source_start; //mapFromSource(source_start);

    beginRemoveRows(QModelIndex(), start, end);

    for(int i=start; i <= end; ++i)
        m_events.removeAt(i);

    endRemoveRows();
}

void ConversationModel::onDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
    // Map the values to our view. Remember the values are inverted
    int tlRow = topLeft.row(); //mapFromSource(bottomRight.row());
    int brRow = bottomRight.row(); //mapFromSource(topLeft.row());

    // Check if this function is safe!
    for(int i=tlRow; i <= brRow; ++i) {
        QModelIndex idx = topLeft.sibling(i, 0);
        m_events[i] = m_model.event(idx);
    }

    QModelIndex tl = index(tlRow, 0);
    QModelIndex br = index(brRow, 0);
    emit dataChanged(tl, br);
}
