// Local includes
#include "conversationmodel.h"

// Global includes
#include <QThread>
#include <QDebug>
#include <QDateTime>

struct Event
{
    Event()
    : status(ConversationModel::DeliveredStatus)
    , isRead(true)
    {
    }

    QString text;
    ConversationModel::EventStatus status;
    QString direction;
    QDateTime endTime;
    QString messageToken;
    int eventId;
    bool isRead;
};

class ConversationModelPrivate
{
public:
    ConversationModelPrivate()
    : groupId(-1)
    {
    }

    ConversationModel* self;

    // Properties
    int groupId;

    QList<Event> events;

    // Functions
    void getEvents();
};

void ConversationModelPrivate::getEvents()
{
    // Don't continue without a group id
    if ( groupId <= 0 )
        return;

    self->beginResetModel();
    events.clear();

    // TODO: Insert data

    self->endResetModel();

    // Change the loading status
    self->setStatus(ConversationModel::Ready);
}

ConversationModel::ConversationModel(QObject *parent)
: QAbstractListModel(parent)
, d(new ConversationModelPrivate)
, m_status(Unknown)
, m_initialized(false)
{
    d->self = this;

    QHash<int, QByteArray> roleNames;
    roleNames.insert(TextRole, "text");
    roleNames.insert(StatusRole, "status");
    roleNames.insert(StatusTextRole, "statusText");
    roleNames.insert(DirectionRole, "direction");
    roleNames.insert(EndTimeRole, "endTime");
    roleNames.insert(MessageTokenRole, "messageToken");
    roleNames.insert(EventIdRole, "eventId");
    roleNames.insert(IsReadRole, "isRead");

    setRoleNames(roleNames);
}

ConversationModel::~ConversationModel()
{
    delete d;
}

void ConversationModel::setGroupId(int groupId)
{
    if ( d->groupId == groupId )
        return;

    d->groupId = groupId;
    emit groupIdChanged();

    if ( m_initialized ) {
        setStatus(Loading);

        d->getEvents();
    }
}

int ConversationModel::groupId() const
{
    return d->groupId;
}

void ConversationModel::setObservedConversation(const QString &observedConversation)
{
    /*if ( observedConversation.isEmpty() )
        m_observedConversation->unsetValue();
    else {
        QString localId = "/org/freedesktop/Telepathy/Account/ring/tel/ring";
        QString remoteId = observedConversation;

        QVariantList value;
        value << QVariant::fromValue(localId) // LocalId
              << QVariant::fromValue(remoteId) // RemoteId
              << QVariant::fromValue(0); // ChatType::ChatType_P2P

        m_observedConversation->setValue( QVariant::fromValue(value) );

        // Mark all messages in this conversation as read
        markReadMessages();
    }*/

    emit observedConversationChanged();
}

QString ConversationModel::observedConversation() const
{
    return QString();
}

void ConversationModel::setStatus(ConversationModel::Status status)
{
    if ( m_status == status )
        return;

    m_status = status;
    emit statusChanged();
}

void ConversationModel::markReadMessages()
{
    /*QList<CommHistory::Event> unreadEvents;

    // Mark unread messages and decide if we should play a sound for the incoming one
    foreach(CommHistory::Event event, m_events) {
        if ( event.isRead() )
            continue;

        event.setIsRead(true);
        unreadEvents << event;
    }

    if ( unreadEvents.isEmpty() == false ) {
        m_model.modifyEvents(unreadEvents);
        qDebug() << "Marked" << unreadEvents.size() << "messages as read.";
    }*/
}

ConversationModel::Status ConversationModel::status() const
{
    return m_status;
}

bool ConversationModel::deleteEvent(int eventId)
{
    //return m_model.deleteEvent(eventId);
    return false;
}

void ConversationModel::classBegin()
{
}

void ConversationModel::componentComplete()
{
    m_initialized = true;

    if ( m_groupId != -1 ) {
        // Adjust the status
        setStatus(Loading);

        // Start the loading
        d->getEvents();
    }
}

QVariant ConversationModel::data(const QModelIndex &index, int role) const
{
    // Get the event
    const Event& event = d->events.at(index.row());

    switch(role) {
    case TextRole:
        return event.text;

    case StatusRole:
        return DeliveredStatus;

    case StatusTextRole:
        {
            switch(event.status) {
            case SendingStatus:
                //% "Sending"
                return qtTrId("qtn_web2sms_status_sending");

            case SentStatus:
                //% "Sent"
                return qtTrId("qtn_web2sms_status_sent");

            case DeliveredStatus:
                //% "Delivered"
                return qtTrId("qtn_web2sms_status_delivered");

            case TemporarilyFailedStatus:
                //% "Temporarily failed"
                return qtTrId("qtn_web2sms_status_temporarily_failed");

            case PermanentlyFailedStatus:
                //% "Failed"
                return qtTrId("qtn_web2sms_status_permanently_failed");

            case UnknownStatus:
            default:
                //% "Unknown"
                return qtTrId("qtn_web2sms_status_unknown");
            }
            return QString();
        }

    case DirectionRole:
        return event.direction;

    case EndTimeRole:
        {
            QDateTime now = QDateTime::currentDateTime();
            QDateTime dt = event.endTime.toLocalTime();

            QString time = dt.time().toString(Qt::SystemLocaleShortDate);
            if ( dt.date() == now.date() ) {
                return time;
            } else {
                QString date = dt.date().toString(Qt::SystemLocaleShortDate);
                //% "%1 | %2"
                //: "DateTime format in GroupModel. %1 is the Date, %2 is the Time. Normal format is date | time if not the same day"
                return qtTrId("qtn_web2sms_datetime_format").arg(date).arg(time);
            }
        }

    case MessageTokenRole:
        return event.messageToken;

    case EventIdRole:
        return event.eventId;

    case IsReadRole:
        return QVariant::fromValue( event.isRead );

    default:
        return QVariant();
    }

    return QVariant();
}

bool ConversationModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    qDebug() << Q_FUNC_INFO << index << value << roleNames().value(role);
    return false;
}

void ConversationModel::onModelReady(bool success)
{
}

void ConversationModel::onModelReset()
{
}

int ConversationModel::rowCount(const QModelIndex& parent) const
{
    if ( parent.isValid() )
        return 0;

    return d->events.size();
}

void ConversationModel::onRowsInserted(const QModelIndex &, int start, int end)
{
}

void ConversationModel::onRowsRemoved(const QModelIndex &, int source_start, int source_end)
{
}

void ConversationModel::onDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
}
