// Local includes
#include "groupmodel.h"

// Global includes
#include <QContact>
#include <QContactDisplayLabel>
#include <CommHistory/commonutils.h>
#include <QThread>

// Inspiration taken from https://gitorious.org/commhistory/libcommhistory/blobs/master/tools/commhistory-tool.cpp
#define TELEPATHY_ACCOUNT_PREFIX       QLatin1String("/org/freedesktop/Telepathy/Account/")
#define TELEPATHY_RING_ACCOUNT_POSTFIX QLatin1String("ring/tel/ring")
#define RING_ACCOUNT TELEPATHY_ACCOUNT_PREFIX + TELEPATHY_RING_ACCOUNT_POSTFIX

GroupModel::GroupModel(QObject *parent)
: CommHistory::GroupModel(parent)
, m_contactManager("tracker")
{
    // Hackity
    connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SIGNAL(countChanged()));
    connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SIGNAL(countChanged()));

    QHash<int, QByteArray> roleNames;
    roleNames.insert(GroupIdRole, "groupId");
    roleNames.insert(ContactNameRole, "contactName");
    roleNames.insert(ContactIdRole, "contactId");
    roleNames.insert(LastMessageTextRole, "lastMessageText");
    roleNames.insert(EndTimeRole, "endTime");
    roleNames.insert(RemoteUidRole, "remoteUid");
    roleNames.insert(SectionRole, "section");
    roleNames.insert(UnreadMessagesRole, "unreadMessages");
    roleNames.insert(LastEventStatusRole, "lastEventStatus");

    setRoleNames(roleNames);

    QThread* thread = new QThread(this);
    thread->start();
    setBackgroundThread(thread);
}

GroupModel::~GroupModel()
{
    QThread* thread = backgroundThread();
    thread->terminate();
    thread->wait();
}

void GroupModel::classBegin()
{
}

void GroupModel::componentComplete()
{
    // Update the model
    getGroups(RING_ACCOUNT, m_remoteUid);
}

QVariant GroupModel::get(int row) const
{
    // Sanity check
    QModelIndex indexOfRow = index(row, 0);
    if ( indexOfRow.isValid() == false )
        return QVariant();

    // Create a small map that returns the section
    QVariantMap map;
    map.insert("section", indexOfRow.data(SectionRole));

    return QVariant::fromValue(map);
}

int GroupModel::count() const
{
    return rowCount();
}

void GroupModel::setRemoteUid(const QString& remoteUid)
{
    if ( m_remoteUid == remoteUid )
        return;

    m_remoteUid = remoteUid;
    emit remoteUidChanged();
}

QString GroupModel::remoteUid() const
{
    return m_remoteUid;
}

QVariant GroupModel::data(int row, int role) const
{
    return data(index(row, 0), role);
}

int GroupModel::rowForRemoteUid(const QString &remoteUid)
{
    int rows = rowCount();
    for(int i=0; i < rows; ++i) {
        QModelIndex index = this->index(i, 0);
        CommHistory::Group group = this->group(index);

        foreach(const QString& tmp, group.remoteUids()) {
            if ( CommHistory::remoteAddressMatch(remoteUid, tmp) )
                return i;
        }
    }

    return -1;
}

int GroupModel::groupIdForRemoteUid(const QString &remoteUid)
{
    int rows = rowCount();
    for(int i=0; i < rows; ++i) {
        QModelIndex index = this->index(i, 0);
        CommHistory::Group group = this->group(index);

        foreach(const QString& tmp, group.remoteUids()) {
            if ( CommHistory::remoteAddressMatch(remoteUid, tmp) )
                return group.id();
        }
    }

    return -1;
}

QVariant GroupModel::data(const QModelIndex &index, int role) const
{
    // Get the group at the given position
    CommHistory::Group group = this->group(index);

    switch(role) {
    case GroupIdRole:
        return QVariant::fromValue(group.id());

    case RemoteUidRole:
        {
            QStringList remoteUids = group.remoteUids();
            if ( remoteUids.isEmpty() )
                return QString();
            else
                return remoteUids.first();
        }

    case ContactIdRole:
        {
            QList<CommHistory::Event::Contact> contacts = group.contacts();
            if ( contacts.isEmpty() )
                return -1;
            else
                return contacts.first().first;
        }

    case ContactNameRole:
        {
            QList<CommHistory::Event::Contact> contacts = group.contacts();
            if ( contacts.isEmpty() ) {
                // Try to get a name from the remote uids
                QString name = group.remoteUids().join(",");
                if ( name.isEmpty() == false )
                    return name;

                // Try the chat name
                name = group.chatName();
                if ( name.isEmpty() == false )
                    return name;

                // Give up ...
                //% "<unknown contact>"
                return qtTrId("qtn_web2sms_unknown_contact");
            }
            else {
                CommHistory::Event::Contact contact = contacts.first();

                // Do the expensive way?
                if ( contact.second.isEmpty() ) {
                    QContact c = m_contactManager.contact(contact.first);

                    QContactDisplayLabel label = c.detail<QContactDisplayLabel>();
                    return label.label();
                }
                else
                    return contact.second;
            }
        }

    case LastMessageTextRole:
        return QVariant::fromValue(group.lastMessageText());

    case UnreadMessagesRole:
        return group.unreadMessages();

    case LastEventStatusRole:
        return group.lastEventStatus();

    case EndTimeRole:
        {
            QDateTime now = QDateTime::currentDateTime();
            QDateTime dt = group.endTime().toLocalTime();

            QString time = dt.time().toString(Qt::SystemLocaleShortDate);
            if ( dt.date() == now.date() ) {
                return time;
            } else {
                QString date = dt.date().toString(Qt::SystemLocaleShortDate);
                //% "%1 | %2"
                //: "DateTime format in GroupModel. %1 is the Date, %2 is the Time. Normal format is date | time if not the same day"
                return qtTrId("qtn_web2sms_datetime_format").arg(date).arg(time);
            }
        }

    case SectionRole:
        {
            // TODO: Calculate the section time
            QDateTime dt = group.endTime().toLocalTime();
            QDateTime now = QDateTime::currentDateTime();

            QDate groupDate = dt.date();
            QDate nowDate = now.date();

            if ( groupDate == nowDate ) {
                //% "Today"
                return qtTrId("qtn_web2sms_section_today");
            } else {
                QDate yesterday = nowDate.addDays(-1);
                if ( groupDate == yesterday ) {
                    //% "Yesterday"
                    return qtTrId("qtn_web2sms_section_yesterday");
                }

                // Calculate difference in weeks
                if ( dt.date().year() == now.date().year() ) {
                    int weeks = nowDate.weekNumber() - groupDate.weekNumber();

                    // Return the name of the day
                    if ( weeks == 0 || weeks == -1 ) {
                        return dt.toString("dddd");
                    } else if ( weeks == 1 ) {
                        //% "Last week"
                        return qtTrId("qtn_web2sms_section_last_week");
                    } else if ( weeks > 0 && weeks < 4 ) {
                        //% "%n weeks ago"
                        return qtTrId("qtn_web2sms_section_weeks_ago", weeks);
                    } else {
                        return dt.toString("MMMM");
                    }
                } else {
                    //% "%1, %2"
                    //: "This is the date formatting for messages a long time ago. %1 is the month like January, %2 is the year."
                    return qtTrId("qtn_web2sms_section_month_year").arg(dt.toString("MMMM")).arg(dt.date().year());
                }
            }
        }

    default:
        return QVariant();
    }
}
