// Local includes
#include "messagehandler.h"

// Global includes
#include <QDebug>
#include <QUuid>
#include <QUrl>
#include <QStringList>
#include <QPluginLoader>
#include <QEventLoop>
#include <QTimer>

#ifdef Q_OS_ANDROID
#include <QJson/Serializer>
#include <QJson/Parser>
#else // Q_OS_ANDROID
#include <qjson/serializer.h>
#include <qjson/parser.h>
#endif // Q_OS_ANDROID
using namespace QJson;

#ifdef MEEGO_EDITION_HARMATTAN
namespace Accounts {
    class Provider;
    class AgProvider;
    class Service;
    class AgService;
    typedef QList<Service*> ServiceList;
}
#include <accounts-qt/account.h>
#include <CommHistory/trackerio.h>
#include <CommHistory/commonutils.h>

using namespace CommHistory;

// Inspiration taken from https://gitorious.org/commhistory/libcommhistory/blobs/master/tools/commhistory-tool.cpp
#define TELEPATHY_ACCOUNT_PREFIX       QLatin1String("/org/freedesktop/Telepathy/Account/")
#define TELEPATHY_RING_ACCOUNT_POSTFIX QLatin1String("ring/tel/ring")
#define RING_ACCOUNT TELEPATHY_ACCOUNT_PREFIX + TELEPATHY_RING_ACCOUNT_POSTFIX

#else // MEEGO_EDITION_HARMATTAN
typedef QObject EventModel;
#endif // MEEGO_EDITION_HARMATTAN

class ModelWatcher : public QObject
{
    Q_OBJECT

public:
    void waitForSignals() {
        m_loop.exec();
    }

    void setModel(EventModel* model) {
        connect(model, SIGNAL(eventsCommitted(QList<CommHistory::Event>,bool)), this, SLOT(didCommitEvents()));
    }

public slots:
    void didCommitEvents() {
        // Hope this time is enough for d-bus to populate the eventsAdded signal
        QTimer::singleShot(50, this, SLOT(didTimeout()));
    }

    void didTimeout() {
        m_loop.exit();
    }

protected:
    QEventLoop m_loop;
};

MessageHandler::MessageHandler(QObject *parent)
: QObject(parent)
, m_onlineStatus(Offline)
{
    // We want the name of the ovi account
    accounts.setProviderFilter("ovi");
    accounts.componentComplete();

#ifdef MEEGO_EDITION_HARMATTAN
    // Setup contact group backend
    m_groupModel.enableContactChanges(false);
    m_groupModel.getGroups(RING_ACCOUNT);

    if ( initializeNotifications() ) {
        qDebug() << "Registering application for notifications";
        session->registerApplication("net.muhkuh.web2sms");
    }
    else
        qDebug() << "Failed registering for notifications";
#endif // MEEGO_EDITION_HARMATTAN
}

MessageHandler::~MessageHandler()
{
#ifdef MEEGO_EDITION_HARMATTAN
    if ( session ) {
        session->unregisterApplication();
        delete session;
    }
#endif // MEEGO_EDITION_HARMATTAN
}

void MessageHandler::setServerAddress(const QString& serverAddress)
{
    if ( m_serverAddress == serverAddress )
        return;

    m_serverAddress = serverAddress;
    emit serverAddressChanged();
}

QString MessageHandler::serverAddress() const
{
    return m_serverAddress;
}

MessageHandler::OnlineStatus MessageHandler::onlineStatus() const
{
    return m_onlineStatus;
}

int MessageHandler::sendMessage(AccountWrapper* account, const QVariantList& contacts, const QString &text)
{
    if ( account == NULL ) {
        qDebug() << __PRETTY_FUNCTION__ << ": No account supplied, stopping";
        return -1;
    }

    // Get an existing group or create a new one
    QVariantMap firstContact = contacts.first().toMap();
    QString number = firstContact.value("number").toString();

    qDebug() << "sendMessage:" << account->providerId() << "number:" << number << "text:" << text;

#ifdef MEEGO_EDITION_HARMATTAN
    int groupId = findGroupForNumber( number );
    if ( groupId == -1 )
        groupId = createGroupWithNumber( number );

    Event event;
    event.setType(Event::SMSEvent);
    event.setDirection(Event::Outbound);
    //event.setIsRead(true);
    event.setMessageToken(QUuid::createUuid().toString());
    event.setStartTime(QDateTime::currentDateTime());
    event.setEndTime(QDateTime::currentDateTime());

    event.setGroupId(groupId);
    event.setLocalUid(RING_ACCOUNT);
    event.setRemoteUid(number);
    event.setFreeText(text);
    event.setStatus(Event::SendingStatus);

    /// TODO: Remove this code as fast as possible, hackity!
    m_eventIdToAccount.insert(event.messageToken(), account);

    // Add the event
    m_eventModel.addEvent(event);

    // Wait before sending. We ensure the event exists before
    // anyone can change the state to Error or something else.
    ModelWatcher watcher;
    watcher.setModel(&m_eventModel);
    watcher.waitForSignals();

    // Really send the message
    remoteSendMessage(event.messageToken(), account, contacts, text);

    return groupId;
#else
    return -1;
#endif // MEEGO_EDITION_HARMATTAN
}

bool MessageHandler::resendMessage(AccountWrapper *account, const QString &messageToken)
{
#ifdef MEEGO_EDITION_HARMATTAN
    if ( account == NULL ) {
        qDebug() << __PRETTY_FUNCTION__ << ": No account supplied, stopping";
        return false;
    }

    // Try to find the event and get some information from it
    Event event;
    if ( false == m_eventModel.trackerIO().getEventByMessageToken(messageToken, event) ) {
        qDebug() << "Can't find message with token" << messageToken << ", stopping.";
        return false;
    }

    // Change the status to sending
    event.setStatus(Event::SendingStatus);

    // Modify the event
    m_eventModel.modifyEvent(event);

    // Prepare the contact information
    QVariantList contacts;
    foreach(const Event::Contact& eventContact, event.contacts()) {
        QVariantMap contact;
        contact.insert("name", eventContact.second);
        contact.insert("number", event.remoteUid());
        contacts.append(contact);
        break; // Since we only have one remoteUid, we can stop here
    }

    /// TODO: Remove this code as fast as possible, hackity!
    m_eventIdToAccount.insert(messageToken, account);

    // Really send the message again
    remoteSendMessage(messageToken, account, contacts, event.freeText());
    return true;
#else
    return false;
#endif // MEEGO_EDITION_HARMATTAN
}

int MessageHandler::createGroupWithNumber(const QString &number)
{
#ifdef MEEGO_EDITION_HARMATTAN
    Group group;
    group.setLocalUid(RING_ACCOUNT);
    group.setRemoteUids( QStringList() << number );

    if ( m_groupModel.addGroup(group) == false ) {
        qCritical() << "Error adding group";
        return -1;
    }

    int groupId = group.id();
    qDebug() << "Added group" << groupId;
    return groupId;
#else
    return -1;
#endif // MEEGO_EDITION_HARMATTAN
}

int MessageHandler::findGroupForNumber(const QString &number) const
{
#ifdef MEEGO_EDITION_HARMATTAN
    int groupId = -1;
    for(int i=0; i < m_groupModel.rowCount(); ++i) {
        QModelIndex index = m_groupModel.index(i, 0);
        Group group = m_groupModel.group(index);

        foreach(const QString& tmp, group.remoteUids()) {
            if ( CommHistory::remoteAddressMatch(number, tmp) ) {
                qDebug() << "Founding existing group" << group.id();
                groupId = group.id();
                break;
            }
        }
    }

    return groupId;
#else
    return -1;
#endif
}

void MessageHandler::remoteSendMessage(const QString &eventId, const AccountWrapper* account, const QVariantList &contacts, const QString &text)
{
#ifdef MEEGO_EDITION_HARMATTAN
    Accounts::Account* oviAccount = (accounts.rowCount() > 0) ? accounts.account(0) : NULL;

    if ( oviAccount == NULL ) {
        qDebug() << "Cannot find ovi account, message can not be sent!";
        return;
    }

    QVariantMap credentials;
    credentials.insert("CLIENT_VERSION", PACKAGEVERSION);
    credentials.insert("PASSWORD", account->password());
    credentials.insert("LOGIN", account->username());
    credentials.insert("NUMBERS", contacts);
    credentials.insert("SMSTEXT", text.trimmed());
    credentials.insert("PROVIDERID", account->providerId());
    credentials.insert("MESSAGETYPE", ""); // TOOD: Fill message type
    credentials.insert("EVENTID", eventId);
    credentials.insert("PUSHID", oviAccount->displayName());
    credentials.insert("PLUGINDICT", account->pluginCredentials());

    // Build the api key for the server
    QUuid apiKey;
    if ( m_serverAddress == API_SERVER ) {
        apiKey = QUuid(API_KEY_L,
                       API_KEY_W1, API_KEY_W2,
                       API_KEY_B1, API_KEY_B2, API_KEY_B3, API_KEY_B4,
                       API_KEY_B5, API_KEY_B6, API_KEY_B7, API_KEY_B8);
    }

    // Remove the brackets
    QString apiString = apiKey.toString();
    apiString.remove('{');
    apiString.remove('}');

    credentials.insert("WEB2SMS_API", apiString);

    // Serialize the data
    Serializer serializer;
    QByteArray data = serializer.serialize(credentials);
    // Don't uncomment this line, it's a security risk!
    //qDebug() << "Serialized data:" << data;

    // Create the network request
    QUrl url(m_serverAddress);
    url.setPath("/send");

    qDebug() << "Using server:" << url;

    QNetworkRequest request(url);
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");

    // Actually send the request
    QNetworkReply* reply = network.post(request, data);
    reply->setProperty("eventId", eventId);
    connect(reply, SIGNAL(finished()), this, SLOT(didFinishRequest()));
#endif // MEEGO_EDITION_HARMATTAN
}

void MessageHandler::didFinishRequest()
{
    // Get the sending reply
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());

    // Get the event id back
    QString eventId = reply->property("eventId").toString();

    if ( reply->error() ) {
        //% "HTTP: %1"
        processError(eventId, qtTrId("qtn_web2sms_messagehandler_http_error").arg(reply->errorString()));
    } else {
        QByteArray data = reply->readAll();

        qDebug() << "Received data:" << data;

        bool ok=false;
        QJson::Parser parser;
        QVariant object = parser.parse(data, &ok);

        if ( ok ) {
            processMessage(eventId, object);
        } else {
            qDebug() << "JSon data is malformed";

            //% "Received invalid JSon data from server"
            processError(eventId, qtTrId("qtn_web2sms_messagehandler_invalid_json_data"));
        }
    }

    // Free memory
    reply->deleteLater();
}

bool MessageHandler::initializeNotifications()
{
#ifdef MEEGO_EDITION_HARMATTAN
    session = NULL;

    QPluginLoader* loader = new QPluginLoader("/usr/lib/qt4/plugins/liboneplugin.so");
    QObject* serviceObject = loader->instance();
    qDebug() << loader->errorString();

    delete loader;

    if ( serviceObject == NULL )
        return false;

    // Get the session handle
    session = static_cast<OviNotificationSession*>(serviceObject);

    // Connect the signals
    connect(session, SIGNAL(stateChanged(QObject*)), this, SLOT(didChangeState(QObject*)));
    connect(session, SIGNAL(received(QObject*)), this, SLOT(didReceiveNotification(QObject*)));

    return true;
#else
    return false;
#endif
}

void MessageHandler::didChangeState(QObject *object)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Get the actual state
    OviNotificationState* state = static_cast<OviNotificationState*>(object);

    switch(state->sessionState()) {
    case OviNotificationState::EStateOffline:
        m_onlineStatus = Offline;
        qDebug() << "Application is offline.";
        break;

    case OviNotificationState::EStateOnline:
        m_onlineStatus = Online;
        qDebug() << "Application is online";
        break;

    case OviNotificationState::EStateConnecting:
        m_onlineStatus = Connecting;
        qDebug() << "Application is connecting";
        break;

    default:
        break;
    }

    if ( state->sessionError() != OviNotificationState::EErrorNone ) {
        m_onlineStatus = Error;
        qDebug() << "Error" << state->sessionErrorString() << "occured!";
    }

    delete state;

    emit onlineStatusChanged();
#endif
}

void MessageHandler::didReceiveNotification(QObject *object)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Get the message
    OviNotificationMessage* message = static_cast<OviNotificationMessage*>(object);

    // Get the payload
    OviNotificationPayload* payload = static_cast<OviNotificationPayload*>(message->payload());
    qDebug() << "Received notification:" << payload->data();

    bool ok=false;
    QJson::Parser parser;
    QVariant data = parser.parse(payload->data(), &ok);

    if ( ok ) {
        processMessage(QString(), data);
    } else {
        qDebug() << "Received invalid JSon data from server, can't find the eventId";
    }

    delete payload;
    delete message;
#endif
}

void MessageHandler::processError(const QString& eventId, const QString &text)
{
    changeMessageStatus(eventId, ::ConversationModel::PermanentlyFailedStatus);

    // Notify that we have an error
    emit error(text);
}

void MessageHandler::processMessage(const QString& eventId, const QVariant &data)
{
    /*
      Typically a message should look like:

      {
          "status" : {
             "TYPE":"Success",                < Mandatory
             "MESSAGE":"some error happened", < Optional
             ""
          },
          "EVENTID":"1234",
          "BALANCE":"123"
      }

      Status types: Success, Delayed, Error

     */

    QString id = eventId;
    if ( data.type() == QVariant::Map ) {
        QVariantMap mapping = data.toMap();

        // Get the eventId either from the supplied parameter or from the message
        id = eventId.isEmpty() ? mapping.value("EVENTID").toString() : eventId;
        QString balance = mapping.value("BALANCE").toString();

        QVariantMap status = mapping.value("STATUS").toMap();
        QString statusType = status.value("TYPE").toString();
        QString statusMessage = status.value("MESSAGE").toString();

        /// FIXME: Hackity code!
        if ( balance.isEmpty() == false ) {
            AccountWrapper* account = m_eventIdToAccount.value(id);
            if ( account ) {
                account->setBalance(balance);
                account->sync();
                qDebug() << "Changed account balance of" << account->displayName() << "to" << balance;
            } else
                qDebug() << "Received balance but didn't find a account for it";
        }

        // Bail out here if we get a delayed answer
        if ( statusType.compare("delayed", Qt::CaseInsensitive) == 0 ) {
            qDebug() << "Server said: Message" << id << "was delayed, waiting for notification.";
            return;
        }

        if ( statusType.compare("error", Qt::CaseInsensitive) == 0 ) {
            qDebug() << "Server said: Message" << id << "failed to send";
            processError(id, statusMessage);
            return;
        }

        if ( statusType.compare("success", Qt::CaseInsensitive) == 0 ) {
            qDebug() << "Server said: Message" << id << "was sent";
            changeMessageStatus(id, ::ConversationModel::SentStatus);
            return;
        }

        //% "Server replied unknown status: %1"
        processError(id, qtTrId("qtn_web2sms_messagehandler_unknown_status").arg(statusType));
        return;
    }

    //% "The server repied a wrong data format"
    processError(id, qtTrId("qtn_web2sms_messagehandler_wrong_data_format"));
}

void MessageHandler::changeMessageStatus(const QString &eventId, ::ConversationModel::EventStatus status)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Try to find the event
    Event event;
    if ( false == m_eventModel.trackerIO().getEventByMessageToken(eventId, event) ) {
        qDebug() << "Can't find message with id" << eventId << ", stopping.";
        return;
    }

    qDebug() << "Message" << eventId << "modified.";

    // Change the status
    event.setStatus( static_cast<CommHistory::Event::EventStatus>(status) );

    // Modify the event
    m_eventModel.modifyEvent(event);
#endif // MEEGO_EDITION_HARMATTAN
}

#include "messagehandler.moc"
