#include "messagestatistics.h"

#include <QDebug>

MessageStatistics::MessageStatistics(QObject *parent)
: QObject(parent)
, m_messageType(NULL)
, m_charset(GSM)
, m_state(Error)
, m_charactersLeft(0)
, m_messageCharacterCount(0)
, m_messageCount(0)
, m_completeCharacterCount(0)
{
    m_messageType = new MessageType("default", "default", QIcon(), "", 160, 3000, 1);
}

void MessageStatistics::recalc()
{
    // Set the default GSM charset
    m_charset = GSM;

    if ( m_messageType ) {
        int completeCharacterCount = 0;

        // Calculate the complete character count
        QString messageText = m_text.trimmed();
        for(int i=0; i < messageText.size(); ++i)
        {
          // Get the two byte code
          unsigned short twoByte = gsmCodec.twoByteFromUnicode(messageText.at(i));

          if ( twoByte == 0x10 ) {
            // Unsupported characters force us to use unicode
            m_charset = Unicode;
            // Now we can take the complete text length
            completeCharacterCount = messageText.size();
            break;
          }
          else if ( twoByte > 256 )
            completeCharacterCount += 2;
          else
            completeCharacterCount++;
        }

        // ### TODO: REMOVE THIS HARDODED COUNTER
        int singleMessageLength = (m_charset == GSM) ? m_messageType->singleMessageLength() : 70;

        // Calculate message information
        int messageCharacterCount = completeCharacterCount % singleMessageLength;
        int messageCount = (completeCharacterCount / singleMessageLength) + 1;

        int remainingCharacters;
        if ( messageCount > 1 && messageCharacterCount == 0 ) {
            remainingCharacters = 0;
            --messageCount;
        }
        else
            remainingCharacters = singleMessageLength - messageCharacterCount;

        // Apply data
        m_completeCharacterCount = completeCharacterCount;
        m_charactersLeft = remainingCharacters;
        m_messageCount = messageCount;
    } else {
        m_completeCharacterCount = 0;
        m_charactersLeft = 0;
        m_messageCharacterCount = 0;
        m_messageCount = 0;
    }

    // Update the states
    m_state = Error;
    if ( m_completeCharacterCount > 0 ) {
        if ( m_completeCharacterCount <= m_messageType->maxMessageLength() ) {
            // Yay, checks seem to have passed

            int receivers = m_contacts.size();
            qDebug() << "MessageStatistics:" << m_contacts << receivers;
            if ( m_messageType->maxReceivers() < receivers ) {
                //% "Maximum receivers is %n"
                m_errorString = qtTrId("qtn_web2sms_messagestatistics_too_many_receivers", m_messageType->maxReceivers());
            } else if ( receivers == 0 ) {
                //% "No receivers"
                m_errorString = qtTrId("qtn_web2sms_messagestatistics_no_receivers");
            } else {
                // TODO: We still need the receivers
                m_state = Ok;
                m_errorString = QString();
            }
        } else {
            //% "Message is too long"
            m_errorString = qtTrId("qtn_web2sms_messagestatistics_too_many_characters");
        }
    }
    else {
        //% "Empty message"
        m_errorString = qtTrId("qtn_web2sms_messagestatistics_no_text");
    }

    emit modified();
}

void MessageStatistics::setText(const QString& text)
{
    if ( m_text == text )
        return;

    m_text = text;
    recalc();
}

QString MessageStatistics::text() const
{
    return m_text;
}

void MessageStatistics::setMessageType(MessageType* messageType)
{
    if ( m_messageType == messageType )
        return;

    m_messageType = messageType;
    recalc();
}

MessageType* MessageStatistics::messageType() const
{
    return m_messageType;
}

MessageStatistics::Charset MessageStatistics::charset() const
{
    return m_charset;
}

void MessageStatistics::setContacts(const QVariantList& contacts)
{
    if ( m_contacts == contacts )
        return;

    m_contacts = contacts;
    recalc();
}

QVariantList MessageStatistics::contacts() const
{
    return m_contacts;
}

MessageStatistics::State MessageStatistics::state() const
{
    return m_state;
}

QString MessageStatistics::errorString() const
{
    return m_errorString;
}

int MessageStatistics::charactersLeft() const
{
    return m_charactersLeft;
}

int MessageStatistics::messageCharacterCount() const
{
    return m_messageCharacterCount;
}

int MessageStatistics::messageCount() const
{
    return m_messageCount;
}

int MessageStatistics::completeCharacterCount() const
{
    return m_completeCharacterCount;
}

void MessageStatistics::classBegin()
{
}

void MessageStatistics::componentComplete()
{
    recalc();
}
