// Local includes
#include "pluginresolver.h"
#include "zipfilesystem.h"

// Global includes
#include <QtCore/QDir>
#include <QtCore/QFile>
#include <QtCore/QDebug>
#include <QtCore/QXmlStreamReader>
#include <QtCore/QMetaEnum>
#include <QtCore/QCoreApplication>
#include <QtCore/QTranslator>
#include <QtGui/QDesktopServices>

#define ZIP_NAME "plugins.zip"
#define DIR_NAME ".web2sms"

class PluginResolverPrivate
{
public:
    PluginResolverPrivate()
    : zipFS(NULL)
    , translator(NULL)
    , pluginTimestamp(0)
    {}

    ~PluginResolverPrivate()
    {
        cleanup();
    }

    PluginResolver* self;

    enum PluginLocation
    {
        UserDirectory = 1,
        SystemDirectory = 2,
        BothDirectories = UserDirectory | SystemDirectory
    };
    Q_DECLARE_FLAGS(PluginLocations, PluginLocation)

    // Properties
    QList<ProviderInfo*> providers;
    ZipFileSystem* zipFS;
    QTranslator* translator;
    int pluginTimestamp;

    // Functions
    QString pluginPath(PluginLocations location = BothDirectories) const;

    void cleanup();
    void loadProviders();
    void loadProvider(const QDir& dir, const QString& fileName);

    QString readElementContent(QXmlStreamReader& xml) const;
};

QString PluginResolverPrivate::pluginPath(PluginLocations location) const
{
    if ( location.testFlag(UserDirectory) ) {
        // Take a look into the user's home directory and make sure there is a web2sms directory
        QDir userDir( QDesktopServices::storageLocation(QDesktopServices::HomeLocation) );
        if ( userDir.exists(DIR_NAME) == false )
            userDir.mkdir(DIR_NAME);
        userDir.cd(DIR_NAME);

        QString filePath = userDir.exists(ZIP_NAME) ? userDir.absoluteFilePath(ZIP_NAME) : QString();
        if ( location == UserDirectory || !filePath.isEmpty() )
            return filePath;
    }

    if ( location.testFlag(SystemDirectory) ) {
        // Yeah, this is really badly hardcoded
        return "/opt/web2sms/" ZIP_NAME;
    }

    // Bad things happen
    return QString();
}

void PluginResolverPrivate::cleanup()
{
    // Clear the providers
    qDeleteAll(providers);
    providers.clear();

    // Delete the translator
    if ( translator ) {
        qApp->removeTranslator(translator);

        delete translator;
        translator = NULL;
    }

    // Delete the zip file system
    if ( zipFS ) {
        zipFS->close();

        delete zipFS;
        zipFS = NULL;
    }
}

void PluginResolverPrivate::loadProviders()
{
    // Cleanup before (re)loading
    cleanup();

    // Get the path for a plugin
    QString pluginPath = this->pluginPath();
    zipFS = new ZipFileSystem(pluginPath);

    // Try to load the plugin
    if ( zipFS->open() == false ) {
        qDebug() << "Unable to open plugin file:" << pluginPath;
        return;
    }

    // Load translations
    QString fileName = QString("plugins_%1").arg( QLocale::system().name() );

    translator = new QTranslator(self);
    // Load the default translation and if it fails, try the english one
    if ( translator->load(fileName, pluginPath) || translator->load("plugins_en", pluginPath) ) {
        qApp->installTranslator(translator);

        qDebug() << "Installed plugin translations";
    }


    // Iterate all subdirectories in the plugins folder
    QDir dir(pluginPath);

    // Load the timestamp file
    QFile timestampFile(dir.absoluteFilePath("timestamp"));
    if ( timestampFile.open(QIODevice::ReadOnly) ) {
        QByteArray value = timestampFile.readLine();
        value.replace('\n', QString());
        value.replace('\r', QString());
        qDebug() << "Timestamp value" << value;
        pluginTimestamp = value.toInt();
        qDebug() << "Plugin timestamp loaded" << pluginTimestamp;
        emit self->pluginTimestampChanged();
    } else {
        qDebug() << "Failed to open timestamp file";
    }

    // Setup a filter for xml files
    QStringList filterNames;
    filterNames << "*.xml";

    foreach(const QString& subDir, dir.entryList(QDir::NoDotAndDotDot | QDir::Dirs)) {
        // Change to the plugin directory
        if ( !dir.cd(subDir) )
            continue;

        // Load all xml files inside the plugin folder
        foreach(const QString& xmlFile, dir.entryList(filterNames, QDir::Files))
            loadProvider(dir, xmlFile);

        // Go one directory up
        dir.cdUp();
    }

    emit self->providersChanged();
}

void PluginResolverPrivate::loadProvider(const QDir &dir, const QString &fileName)
{
    // Try to open the file
    QFile file( dir.filePath(fileName) );
    if ( !file.open(QIODevice::ReadOnly) ) {
        qWarning() << "Error opening provider file " << fileName << ": " << file.errorString();
        return;
    }

    // Setup a provider info structure
    ProviderInfo* info = new ProviderInfo;
    info->setPath(dir.absolutePath());

    // Read the xml contents
    QXmlStreamReader xml(&file);

    while  (!xml.atEnd() ) {
        QXmlStreamReader::TokenType token = xml.readNext();

        if ( token == QXmlStreamReader::StartDocument )
            continue;

        if ( token == QXmlStreamReader::StartElement ) {
            if ( xml.name() == "provider" ) {
                // Get the attributes
                QXmlStreamAttributes attributes = xml.attributes();

                // Read the "id" attribute
                QStringRef refId = attributes.value("id");
                if ( refId.isEmpty() ) {
                    qWarning() << "Provider file " << fileName << " is missing an 'id' attribute. Ignoring file.";
                    return;
                } else {
                    // Remember the id attribute
                    info->setId(refId.toString());
                }

                continue;
            }
            else if ( xml.name() == "name" ) {
                QString idName = readElementContent(xml);
                info->setName( qtTrId(idName.toLocal8Bit()) );
            }
            else if ( xml.name() == "description" ) {
                QString idDescription = readElementContent(xml);
                info->setDescription( qtTrId(idDescription.toLocal8Bit()) );
            }
            else if ( xml.name() == "icon" ) {
                QString icon = readElementContent(xml);

                if ( dir.exists(icon) )
                    info->setIcon( dir.absoluteFilePath(icon) );
                else
                    qWarning() << "Icon file" << dir.filePath(icon) << "does not exist!";
            }
            else if ( xml.name() == "plugin" )
                info->setPlugin( readElementContent(xml) );
            else if ( xml.name() == "accountsetup" ) {
                // Get the file name of the account setup file
                QString fileName = readElementContent(xml);

                // Make sure it exists
                if ( dir.exists(fileName) ) {
                    // And build an absolute path
                    info->setAccountsetup( dir.filePath(fileName) );
                } else {
                    qWarning() << "Accountsetup file" << dir.filePath(fileName) << "does not exist!";
                }
            } else if ( xml.name() == "balanceFormat" ) {
                info->setBalanceFormat( readElementContent(xml) );
            } else if ( xml.name() == "state" ) {
                QString state = readElementContent(xml);
                if ( state.compare("unstable", Qt::CaseInsensitive) == 0 )
                    info->setState( ProviderInfo::Unstable );
                else if ( state.compare("alpha", Qt::CaseInsensitive) == 0 )
                    info->setState( ProviderInfo::Alpha );
                else if ( state.compare("beta", Qt::CaseInsensitive) == 0 )
                    info->setState( ProviderInfo::Beta );
                else if ( state.compare("stable", Qt::CaseInsensitive) == 0 )
                    info->setState( ProviderInfo::Stable );
                else if ( state.compare("broken", Qt::CaseInsensitive) == 0 ) {
                    qDebug() << "Ignoring" << fileName << ", status is broken.";
                    return;
                }
                else
                    qWarning() << "Unsupported state:" << state;
            } else if ( xml.name() == "unicode" ) {
                QString flag = readElementContent(xml);
                info->setSupportsUnicode( flag.compare("yes", Qt::CaseInsensitive) == 0 );
            } else if ( xml.name() == "features") {
                // Read the features
                QString features = readElementContent(xml);

                // Do not handle empty strings
                if ( features.isEmpty() )
                    continue;

                // Get meta information about the flags
                const QMetaObject* meta = info->metaObject();

                // Safety check
                int idx = meta->indexOfEnumerator("ProviderFlag");
                if ( idx == -1 ) {
                    qDebug() << "Can't find ProviderFlag enumerator in MetaObject!";
                    continue;
                }

                // Get the enumerator
                QMetaEnum metaEnum = meta->enumerator(idx);
                ProviderInfo::ProviderFlags oldFlags = info->flags();
                info->setFlags( oldFlags | static_cast<ProviderInfo::ProviderFlags>(metaEnum.keysToValue(features.toLatin1())) );
            } else {
                qWarning() << "Unsupported tag:" << xml.name();
            }
        }
    }

    // Error occurred?
    if ( xml.hasError() ) {
        qWarning() << "Error reading provider file " << fileName << ": " << xml.errorString();
        delete info;
    } else {
        // Append the providers
        providers.append(info);
    }

    // TODO: Error checking etc.
}

QString PluginResolverPrivate::readElementContent(QXmlStreamReader &xml) const
{
    // TODO: Error outputs?
    if ( xml.tokenType() != QXmlStreamReader::StartElement )
        return QString();

    // Ignore the element name
    xml.readNext();

    if ( xml.tokenType() != QXmlStreamReader::Characters )
        return QString();

    // Return the contents
    return xml.text().toString();
}

static PluginResolver* globalPluginResolverInstance = NULL;

static void cleanup_pluginResolver()
{
    delete globalPluginResolverInstance;
}

PluginResolver::PluginResolver(QObject *parent)
: QObject(parent)
, d(new PluginResolverPrivate)
{
    d->self = this;
}

PluginResolver::~PluginResolver()
{
    delete d;
}

void PluginResolver::resolvePlugins()
{
    // Cleanup and load again
    d->cleanup();

    // Load the providers
    d->loadProviders();
}

void PluginResolver::unload()
{
    d->cleanup();
}

QList<ProviderInfo*> PluginResolver::providers() const
{
    return d->providers;
}

QDeclarativeListProperty<ProviderInfo> PluginResolver::providersDeclarative()
{
    return QDeclarativeListProperty<ProviderInfo>(this, d->providers);
}

PluginResolver * PluginResolver::instance()
{
    if ( globalPluginResolverInstance == NULL ) {
        globalPluginResolverInstance = new PluginResolver;
        qAddPostRoutine(cleanup_pluginResolver);
    }

    return globalPluginResolverInstance;
}

ProviderInfo * PluginResolver::providerById(const QString &id) const
{
    foreach(ProviderInfo* info, d->providers)
    {
        if ( info->id() == id )
            return info;
    }

    return NULL;
}

int PluginResolver::pluginTimestamp() const
{
    return d->pluginTimestamp;
}
