// Local includes
#include "zipfilesystem.h"

// Global includes
#include <QFSFileEngine>
#include <QAbstractFileEngineHandler>
#include <QAbstractFileEngineIterator>
#include <QListIterator>
#include <QDebug>
#include <quazip.h>
#include <quazipfile.h>

typedef QSharedPointer<struct Node> NodePtr;

// Small info structure
struct Node {
    QString name;
    QuaZipFileInfo info;

    QAbstractFileEngine::FileFlag flag;

    QList<NodePtr> children;
};

class CustomIterator : public QAbstractFileEngineIterator
{
public:
    CustomIterator(const QStringList& entries, QDir::Filters filters, const QStringList &nameFilters)
    : QAbstractFileEngineIterator(filters, nameFilters)
    , entries(entries)
    , iter(entries)
    {
    }

    bool hasNext() const
    {
        return iter.hasNext();
    }

    QString next()
    {
        return current=iter.next();
    }

    QString currentFileName() const
    {
        return current;
    }

private:
    QStringList entries;
    QListIterator<QString> iter;
    QString current;
};

class ZipFileSystemDirectory : public QAbstractFileEngine
{
public:
    ZipFileSystemDirectory(QuaZip* zip, const QString& file, const QString& absoluteFile, const NodePtr& node)
    : zip(zip)
    , file(file)
    , absoluteFile(absoluteFile)
    , node(node)
    {
    }

    void setFileName(const QString &file) {
        absoluteFile = file;
    }

    QString fileName(FileName fn) const {
        switch(fn) {
        case BaseName:
            {
                QStringList parts = absoluteFile.split('/');
                return parts.last();
            }

        case AbsolutePathName:
            {
                QStringList parts = absoluteFile.split('/');
                parts.removeLast();
                return parts.join("/");
            }

        default:
            return absoluteFile;
        }
    }

    FileFlags fileFlags(FileFlags /*type*/) const {
        if ( node.isNull() )
            return NULL;

        // Base class decides about flags
        FileFlags flags = node->flag;

        flags |= ExistsFlag;
        flags |= ReadOtherPerm | ReadGroupPerm | ReadUserPerm | ReadOwnerPerm;
        flags |= LocalDiskFlag;

        return flags;
    }

    QDateTime fileTime(FileTime /*time*/) const {
        return node->info.dateTime;
    }

    Iterator* beginEntryList(QDir::Filters filters, const QStringList &filterNames) {
        QStringList entries = entryList(filters, filterNames);
        return new CustomIterator(entries, filters, filterNames);
    }

    QStringList entryList(QDir::Filters filters, const QStringList &filterNames) const {
        Q_UNUSED(filterNames);

        QStringList entries;

        foreach(const NodePtr& dir, node->children) {
            bool isDir = !dir->children.isEmpty();

            if ( filters.testFlag(QDir::Dirs) && isDir )
                entries << dir->name;

            if ( filters.testFlag(QDir::Files) && !isDir )
                entries << dir->name;
        }

        return entries;
    }

    qint64 size() const {
        if ( node )
            return node->info.uncompressedSize;
        else
            return 0;
    }

    bool open(QIODevice::OpenMode openMode) {
        if ( !node )
            return false;

        zip->setCurrentFile(node->info.name);
        zipFile.setZip(zip);
        return zipFile.open(QIODevice::ReadOnly);
    }

    bool close() {
        zipFile.close();
        return true;
    }

    qint64 read(char *data, qint64 maxlen) {
        return zipFile.read(data, maxlen);
    }

    qint64 readLine(char *data, qint64 maxlen) {
        return zipFile.readLine(data, maxlen);
    }

    bool seek(qint64 pos) {
        return zipFile.seek(pos);
    }

    bool atEnd() {
        return zipFile.atEnd();
    }

protected:
    QuaZip* zip;
    QuaZipFile zipFile;

    QString file;
    QString absoluteFile;
    NodePtr node;
};

class ZipFileSystemPrivate : public QAbstractFileEngineHandler
{
public:
    ZipFileSystemPrivate()
    : zipFile(NULL)
    , recursionFlag(false)
    {
    }

    ~ZipFileSystemPrivate()
    {
        delete zipFile;
    }

    // Properties
    QString fileName;
    QuaZip* zipFile;

    NodePtr rootNode;

    bool recursionFlag;

    // Functions
    QAbstractFileEngine* create(const QString &fileName) const;
    void cacheContents();

    NodePtr getOrCreateNodeForPath(const QString& path, bool doCreate);
};

QAbstractFileEngine *ZipFileSystemPrivate::create(const QString &fileName) const
{
    // Do we have some zip content?
    if ( fileName.startsWith(this->fileName) == false )
        return NULL;

    // Is it the zip file directly?
    if ( recursionFlag == false ) {
        NodePtr node = rootNode;

        // Get the relative path name
        QString relativePath = fileName.mid(this->fileName.size()+1);

        // Evil casts ...
        node = const_cast<ZipFileSystemPrivate*>(this)->getOrCreateNodeForPath(relativePath, false);

        return new ZipFileSystemDirectory(zipFile, relativePath, fileName, node);
    } else {
        return new QFSFileEngine(fileName);
    }
}

void ZipFileSystemPrivate::cacheContents()
{
    QFileInfo fileInfo(zipFile->getZipName());

    // Clear the structure on rebuild
    rootNode = NodePtr(new Node);
    rootNode->name = fileInfo.fileName();
    rootNode->flag = QAbstractFileEngine::DirectoryType;

    // Get all files inside the zip
    QList<QuaZipFileInfo> infoList = zipFile->getFileInfoList();

    // Build a readable structure
    foreach(const QuaZipFileInfo& zipFileInfo, infoList) {
        QString name = zipFileInfo.name;

        NodePtr node = getOrCreateNodeForPath(name, true);
        node->info = zipFileInfo;

        // WTF, short code does not work here?!
        if ( name.endsWith('/') )
            node->flag = QAbstractFileEngine::DirectoryType;
        else
            node->flag = QAbstractFileEngine::FileType;
    }
}

int findNodeByName(const NodePtr& node, const QString& name) {
    for(int i=0; i < node->children.size(); ++i) {
        if ( node->children.at(i)->name == name )
            return i;
    }

    return -1;
}

NodePtr ZipFileSystemPrivate::getOrCreateNodeForPath(const QString &path, bool doCreate)
{
    // Split the path by parts
    QStringList parts = path.split('/', QString::SkipEmptyParts);

    NodePtr node = rootNode;

    while ( !parts.isEmpty() ) {
        QString dir = parts.takeFirst();

        int pos = findNodeByName(node, dir);
        if ( pos == -1 ) {
            // If creation is not allowed, do nothing
            if ( doCreate == false )
                return NodePtr();

            // Create a new node if we don't find the right one
            NodePtr newNode(new Node);
            newNode->name = dir;
            newNode->flag = QAbstractFileEngine::DirectoryType;
            node->children.append(newNode);

            node = newNode;
        } else {
            node = node->children.at(pos);
        }
    }

    return node;
}

ZipFileSystem::ZipFileSystem(const QString &fileName)
: d(new ZipFileSystemPrivate)
{
    d->fileName = fileName;
    d->zipFile = new QuaZip(fileName);
}

ZipFileSystem::~ZipFileSystem()
{
    delete d;
}

QString ZipFileSystem::fileName() const
{
    return d->fileName;
}

bool ZipFileSystem::open()
{
    // Open the zip file and cache the contents
    d->recursionFlag = true;
    bool success = d->zipFile->open(QuaZip::mdUnzip);

    if ( success )
        d->cacheContents();

    d->recursionFlag = false;

    return success;
}

void ZipFileSystem::close()
{
    d->zipFile->close();
}
