import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Page {
    id: conversationPage

    property string contactName
    property string remoteUid
    property int contactId
    property int groupId

    anchors.margins: UIConstants.DEFAULT_MARGIN

    function updateRingtoneAndVibrate() {
        if ( platformWindow.active && (status==PageStatus.Activating || status==PageStatus.Active) ) {
            console.log("Ignoring notifications for conversation", remoteUid);
            conversationView.remoteUid = remoteUid;
        } else {
            console.log("Activated notifications for all conversations");
            conversationView.remoteUid = "";
        }
    }

    function openContactCard() {
        console.log("Opening contact card");
        picker.openContactCard(conversationPage.contactId);
    }

    function reallySendMessage() {
        // Get the text and numbers
        var account = providerInfoPanel.account;
        var text = messageInput.realText;
        var contacts = messageInfoPanel.statistics.contacts;

        // Actually try to send the message
        backend.messageHandler.sendMessage(account, contacts, text);

        // Clean the text
        messageInput.text = "";
    }

    function sendMessage() {
        // Safety check
        if ( providerInfoPanel.account == null ) {
            //% "The selected provider is invalid"
            banner.showBanner(qsTrId("qtn_web2sms_invalid_provider"));
            return;
        }

        if ( backend.messageHandler.onlineStatus != MessageHandler.Online ) {
            //% "Your phone is offline. Please make sure your Nokia account is properly working."
            banner.showBanner(qsTrId("qtn_web2sms_error_offline"));
            return;
        }

        var contacts = [{"name":contactName, "number":remoteUid}];

        // Check the message
        var statistics = messageInfoPanel.statistics;
        statistics.contacts = contacts;
        if ( statistics.state == MessageStatistics.Error ) {
            banner.showBanner(statistics.errorString);
            return;
        }

        var provider = backend.plugins.providerById(providerInfoPanel.account.providerId);

        // Is this a Unicode message on a non-unicode provider?
        if ( statistics.charset == MessageStatistics.Unicode && provider.supportsUnicode == false ) {
            //% "The selected provider does not support unicode messages"
            banner.showBanner(qsTrId("qtn_web2sms_unicode_not_supported"));
            return;
        }

        if ( provider.state == ProviderInfo.Beta ) {
            if ( appSettings.showBetaWarning )  {
                console.log("Provider is unstable, warning user");
                betaWarning.open();
                return; // DON'T SEND THE MESSAGE NOW
            } else {
                console.log("Provider is unstable, user ignores warnings");
            }
        } else {
            console.log("Provider is stable, sending message");
        }

        // Yeah, really send
        reallySendMessage();
    }

    function resendMessage(messageToken, messageText) {
        console.log("Resend message", messageToken);

        // Safety check
        if ( providerInfoPanel.account == null ) {
            //% "The selected provider is invalid"
            banner.showBanner(qsTrId("qtn_web2sms_invalid_provider"));
            return;
        }

        if ( backend.messageHandler.onlineStatus != MessageHandler.Online ) {
            //% "Your phone is offline. Please make sure your Nokia account is properly working."
            banner.showBanner(qsTrId("qtn_web2sms_error_offline"));
            return;
        }

        // This is really evil, we save the user's text and then set it again ...
        var oldText = messageInput.text;
        messageInput.text = messageText;

        var statistics = messageInfoPanel.statistics;
        var provider = backend.plugins.providerById(providerInfoPanel.account.providerId);

        // Is this a Unicode message on a non-unicode provider?
        if ( statistics.charset == MessageStatistics.Unicode && provider.supportsUnicode == false ) {
            //% "The selected provider does not support unicode messages"
            banner.showBanner(qsTrId("qtn_web2sms_unicode_not_supported"));
            return;
        }

        // Do really resend the message
        backend.messageHandler.resendMessage(providerInfoPanel.account, messageToken);

        messageInput.text = oldText;
    }

    Banner {
        id: banner
        timerShowTime: 10000
    }

    BetaWarning {
        id: betaWarning
        // Send on both, accept and reject
        onAccepted: reallySendMessage()
        onRejected: reallySendMessage()
    }

    ContactPicker {
        id: picker
    }

    // Handle platform window changes
    Connections {
        target: platformWindow
        onActiveChanged: updateRingtoneAndVibrate()
    }

    // Handle page states
    onStatusChanged: updateRingtoneAndVibrate()

    TopToolbar {
        id: header
        anchors {
            left: parent.left
            leftMargin: -UIConstants.DEFAULT_MARGIN
            right: parent.right
            rightMargin: -UIConstants.DEFAULT_MARGIN
            top: parent.top
            topMargin: -UIConstants.DEFAULT_MARGIN
        }

        // Hide the top bar if the software keyboard is visible and we're in landscape
        visible: !(screen.currentOrientation == Screen.Landscape && inputContext.softwareInputPanelVisible)

        ToolButton {
            id: messagesButton
            //% "Messages"
            text: qsTrId("qtn_web2sms_conversationpage_messages")

            anchors.verticalCenter: parent.verticalCenter
            x: 10
            width: 150 // TODO: Don't use a fixed width!

            onClicked: pageStack.pop()
        }

        MouseArea {
            anchors {
                left: contactLabel.left
                right: contactImage.right
                top: parent.top
                bottom: parent.bottom
            }

            onClicked: openContactCard()
        }

        Label {
            id: contactLabel
            anchors {
                left: messagesButton.right
                leftMargin: 8
                right: contactImage.left
                rightMargin: 8
                verticalCenter: parent.verticalCenter
            }

            text: conversationPage.contactName
            horizontalAlignment: Text.AlignRight
            elide: Text.ElideRight
        }

        AvatarImage {
            id: contactImage
            anchors {
                right: parent.right
                rightMargin: 8
                verticalCenter: parent.verticalCenter
            }

            contactId: conversationPage.contactId
        }
    }

    ProviderInfoPanel {
        id: providerInfoPanel
        anchors {
            left: parent.left
            leftMargin: -UIConstants.DEFAULT_MARGIN
            right: parent.right
            rightMargin: -UIConstants.DEFAULT_MARGIN
            top: header.bottom
        }
        visible: header.visible
    }

    ConversationView {
        id: conversationView
        anchors {
            left: parent.left
            right: parent.right
            top: header.visible ? providerInfoPanel.bottom : parent.top
            bottom: column.top
            topMargin: UIConstants.DEFAULT_MARGIN
            bottomMargin: UIConstants.DEFAULT_MARGIN
        }

        groupId: conversationPage.groupId
        onResendMessage: conversationPage.resendMessage(messageToken, messageText)
    }

    Column {
        id: column
        spacing: 2

        anchors {
            left: parent.left
            leftMargin: -UIConstants.DEFAULT_MARGIN
            right: parent.right
            rightMargin: -UIConstants.DEFAULT_MARGIN
            bottom: parent.bottom
            bottomMargin: -UIConstants.DEFAULT_MARGIN
        }

        Rectangle {
            id: warningRect
            width: parent.width
            height: warningLabel.height
            focus: true
            color: "white"

            Connections {
                target: conversationPage
                onRemoteUidChanged: {
                    warningRect.visible = !backend.isInternationalNumber(conversationPage.remoteUid);
                }
            }

            Label {
                id: warningLabel
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
                color: "red"
                wrapMode: Text.Wrap
                //% "You can't reply. Tap here to see why."
                text: qsTrId("qtn_web2sms_conversationpage_no_international_number")
            }

            QueryDialog {
                id: warningDialog
                //% "No international number"
                titleText: qsTrId("qtn_web2sms_no_international_number_title");
                //% "To send a message to this contact, you have to tap on the contact image on top of the conversation page and change the number to a +xx or +00 prefixed format. Then you have to start a new conversation with that contact. Your existing conversation wil be reused."
                message: qsTrId("qtn_web2sms_no_international_number_message");
                //% "Close"
                acceptButtonText: qsTrId("qtn_web2sms_about_close")
            }

            MouseArea {
                anchors.fill: parent
                onClicked: warningDialog.open()
            }
        }

        // Text input
        MessageInputPanel {
            id: messageInput
            width: parent.width
            visible: !warningRect.visible
        }

        // Information about current message
        MessageInfoPanel {
            id: messageInfoPanel
            width: parent.width
            visible: (screen.keyboardOpen || inputContext.softwareInputPanelVisible)
            text: messageInput.realText
            onSendClicked: sendMessage()
        }
    }
}
