import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Page {
    id: newMessagePage

    property string text
    property bool isForwarded: false

    function reallySendMessage() {
        // Get the text and numbers
        var account = providerInfoPanel.account;
        var text = textArea.realText;
        var contacts = receiverArea.contacts;

        // Actually try to send the message
        var groupId = backend.messageHandler.sendMessage(account, contacts, text);
        var conv = groupPage.openConversationWithRemoteUid(contacts[0].number, false);

        // Expect errors :)
        if ( conv == null ) {
            //% "UI Error"
            banner.showBanner( qsTrId("qtn_web2sms_ui_error") );
            return;
        }

        if ( isForwarded )
            pageStack.pop();
        else {
            pageStack.replace(conv);
        }
    }

    function sendMessage() {
        // Safety check
        if ( providerInfoPanel.account == null ) {
            //% "The selected provider is invalid"
            banner.showBanner(qsTrId("qtn_web2sms_invalid_provider"));
            return;
        }

        if ( backend.messageHandler.onlineStatus != MessageHandler.Online ) {
            //% "Your phone is offline. Please make sure your Nokia account is properly working."
            banner.showBanner(qsTrId("qtn_web2sms_error_offline"));
            return;
        }

        // Check the message
        var statistics = infoPanel.statistics;
        if ( statistics.state == MessageStatistics.Error ) {
            banner.showBanner(statistics.errorString);
            return;
        }

        var provider = backend.plugins.providerById(providerInfoPanel.account.providerId);

        // Is this a Unicode message on a non-unicode provider?
        if ( statistics.charset == MessageStatistics.Unicode && provider.supportsUnicode == false ) {
            //% "The selected provider does not support unicode messages"
            banner.showBanner(qsTrId("qtn_web2sms_unicode_not_supported"));
            return;
        }

        if ( provider.state == ProviderInfo.Beta ) {
            if ( appSettings.showBetaWarning )  {
                console.log("Provider is unstable, warning user");
                betaWarning.open();
                return; // DON'T SEND THE MESSAGE NOW
            } else {
                console.log("Provider is unstable, user ignores warnings");
            }
        } else {
            console.log("Provider is stable, sending message");
        }

        // Yeah, really send
        reallySendMessage();
    }

    Banner {
        id: banner
        timerShowTime: 10000
    }

    BetaWarning {
        id: betaWarning
        // Send on both, accept and reject
        onAccepted: reallySendMessage()
        onRejected: reallySendMessage()
    }

    onStatusChanged: {
        // Show the keyboard when the page gets active
        if ( status == PageStatus.Active ) {
            receiverArea.forceActiveFocus();
        }
    }

    TopToolbar {
        id: header
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
        }

        // Hide the top bar if the software keyboard is visible and we're in landscape
        visible: !(screen.currentOrientation == Screen.Landscape && inputContext.softwareInputPanelVisible)

        ToolButton {
            //% "Cancel"
            text: qsTrId("qtn_web2sms_cancel")

            anchors.verticalCenter: parent.verticalCenter
            x: 10
            width: 150 // TODO: Don't use a fixed width!

            onClicked: pageStack.pop()
        }

        Row {
            anchors {
                right: parent.right
                rightMargin: 8
                top: parent.top
                bottom: parent.bottom
            }

            spacing: 8

            Label {
                //% "New SMS"
                text: qsTrId("qtn_web2sms_new_sms")
                anchors.verticalCenter: parent.verticalCenter
            }
        }
    }

    TopToolbar {
        id: contactHeader
        anchors {
            top: header.visible ? header.bottom : parent.top
            left: parent.left
            right: parent.right
        }

        height: Math.max(receiverArea.height, 60)
        source: theme.inverted ? "image://theme/meegotouch-tab-portrait-top-inverted-background" :
                                 "image://theme/meegotouch-tab-portrait-top-background"

        // Center the "To:" label
        Component.onCompleted: {
            toLabel.y = (height/2) - (toLabel.height/2);
            editorImage.y = (height/2) - (editorImage.height/2);
        }

        Image {
            id: editorImage

            anchors {
                left: parent.left
            }

            source: theme.inverted ? "image://theme/meegotouch-editor-inputfield-panel-inverted-background-selected" :
                                     "image://theme/meegotouch-editor-inputfield-panel-background-selected"
            visible: receiverArea.activeFocus
        }

        Label {
            id: toLabel
            anchors {
                top: parent.top
                topMargin: 16
                left: parent.left
                leftMargin: 16
            }

            //% "To:"
            text: qsTrId("qtn_web2sms_newmessagepage_to")
        }

        ContactTextEdit {
            id: receiverArea
            anchors {
                top: parent.top
                left: toLabel.right
                leftMargin: 8
                right: addButton.left
                //right: parent.right
                rightMargin: 8
            }

            //% "Add contacts"
            placeholderText: qsTrId("qtn_web2sms_newmessagepage_add_contacts")
        }

        // NOTE: We don't show the "+" button as long as the reparenting bug of the contacts selector can't be fixed
        ContactChooserSheet {
            id: contactChooser

            onAccepted: {
                var contact = contactChooser.contact;
                console.log("ContactChooserSheet has chosen:", contact);
                receiverArea.tryAddContact(contact);
            }
        }

        Button {
            id: addButton
            anchors {
                top: parent.top
                topMargin: 4
                right: parent.right
                rightMargin: 8
            }
            Behavior on opacity {
                NumberAnimation { duration: 250 }
            }
            opacity: receiverArea.activeFocus ? 1.0 : 0.0
            width: height
            iconSource: theme.inverted ? "image://theme/icon-m-toolbar-add-white" :
                                         "image://theme/icon-m-toolbar-add"
            onClicked: {
                // Force close the keyboard
                contactChooser.forceActiveFocus();
                // And open the sheet
                contactChooser.open()
            }
        }
    }

    ProviderInfoPanel {
        id: providerInfoPanel
        anchors {
            left: parent.left
            right: parent.right
            top: contactHeader.bottom
        }
        visible: header.visible
    }

    Rectangle {
        anchors {
            margins: -16
            fill: textArea
        }

        color: "white"
    }

    PlainTextArea {
        id: textArea

        anchors {
            left: parent.left
            right: parent.right
            top: header.visible ? providerInfoPanel.bottom : contactHeader.bottom
            topMargin: 8
            bottom: infoPanel.top
            bottomMargin: 8
        }

        text: newMessagePage.text
        //% "Write message"
        placeholderText: qsTrId("qtn_web2sms_newmessagepage_text_placeholder")
    }

    // Information about current message
    MessageInfoPanel {
        id: infoPanel
        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        // Assign the message text
        text: textArea.realText
        onSendClicked: sendMessage()
        contacts: receiverArea.contacts
    }
}
