// Local includes
#include "contacttexteditformatter.h"

// Global includes
#include <QDebug>
#include <QTextDocument>
#include <QTextCursor>
#include <QTextBlock>
#include <QDesktopServices>
#include <QDir>
#include <QPainter>
#include <QPixmap>
#include <QRegExp>

enum ResolveState
{
    Unresolved,
    Resolved
};

class ContactTextEditFormatterPrivate : public QObject
{
    Q_OBJECT

public:
    ContactTextEditFormatterPrivate()
    : numberOfContacts(0)
    {
    }

    // Properties
    QFont defaultFont;

    QString outputText;
    QString contactName;

    QString inputText;
    int cursorPosition;

    // Internal properties
    QString plainText;
    QString trimmedText;
    int numberOfContacts;

    // A mapping of image paths to remote uid path
    QHash<QString, QVariantMap> hashPathToRemoteUid;
    QVariantList contacts;

    // Functions
    QString saveContactImage(const QString& text, ResolveState state);
};

QString ContactTextEditFormatterPrivate::saveContactImage(const QString &text, ResolveState state)
{
    // TODO: Maybe should get the cache location
    QDir path(QDesktopServices::storageLocation(QDesktopServices::TempLocation));

    QString stateString;
    switch(state) {
    case Unresolved:
        stateString = "unresolved";
        break;

    case Resolved:
        stateString = "resolved";
        break;
    }

    // Concatenate the file name
    QString fileName = QString("web2sms-cie-%1-%2.png").arg(text).arg(stateString);

    if ( path.exists(fileName) )
        return path.absoluteFilePath(fileName);

    // Calculate the text size
    QFontMetrics metrics(defaultFont);
    QRect textRect = metrics.boundingRect(text);
    textRect.setHeight(32);
    textRect.setWidth( textRect.width() + 16 );

    // Create a new pixmap
    QPixmap pixmap(textRect.size());

    // Draw a border image
    QPixmap borderPixmap( QString("/usr/share/themes/blanco/meegotouch/images/messaging-ui/meegotouch-messaging-recipient-bubble-%1-normal.png").arg(stateString) );
    if ( borderPixmap.isNull() ) {
        borderPixmap = QPixmap(32, 32);
        borderPixmap.fill( state==Unresolved ? "red" : "blue" );
    }

    // Draw the text into the pixmap
    QPainter painter;
    painter.begin(&pixmap);
    //qDrawBorderPixmap(&painter, textRect, QMargins(4,4,4,4), borderPixmap);
    painter.drawPixmap(0, 0, textRect.width(), 32, borderPixmap);

    painter.setPen(Qt::black);
    painter.setFont(defaultFont);
    painter.drawText(0, 0, textRect.width(), 32, Qt::AlignCenter, text);
    painter.end();

    // Save the pixmap
    QString absolutePath = path.absoluteFilePath(fileName);
    pixmap.save(absolutePath);

    return absolutePath;
}

ContactTextEditFormatter::ContactTextEditFormatter(QObject *parent)
: QObject(parent)
, d(new ContactTextEditFormatterPrivate)
{
}

ContactTextEditFormatter::~ContactTextEditFormatter()
{
    delete d;
}

void ContactTextEditFormatter::setDefaultFont(const QFont& font)
{
    if ( d->defaultFont == font )
        return;

    d->defaultFont = font;
    emit defaultFontChanged();
}

QFont ContactTextEditFormatter::defaultFont() const
{
    return d->defaultFont;
}

QString ContactTextEditFormatter::outputText() const
{
    return d->outputText;
}

QString ContactTextEditFormatter::contactName() const
{
    return d->contactName;
}

QVariantList ContactTextEditFormatter::contacts() const
{
    return d->contacts;
}

bool ContactTextEditFormatter::isEmpty() const
{
    return d->plainText.isEmpty() && d->contacts.isEmpty();
}

void ContactTextEditFormatter::setInputData(const QString &inputText, int cursorPosition)
{
    // Sanity check
    if ( d->inputText == inputText && d->cursorPosition == cursorPosition )
        return;

    // Update internal data
    d->inputText = inputText;
    d->cursorPosition = cursorPosition;

    QTextDocument document;
    document.setHtml(inputText);
    document.setDefaultFont(d->defaultFont);

    // Get the trimmed text
    d->plainText = document.toPlainText();
    d->numberOfContacts = 0;

    QChar specialChar(ushort(65532));
    int charPos = d->plainText.indexOf(specialChar);
    while ( charPos != -1 ) {
        d->plainText.remove(charPos, 1);
        ++(d->numberOfContacts);
        charPos = d->plainText.indexOf(specialChar);
    }

    // Trim the text
    d->trimmedText = d->plainText.trimmed();
    d->contacts.clear();

    // Find the contacts in the text
    QRegExp exp("src=\"([^\"]*)\"");
    int pos = exp.indexIn(inputText);
    while ( pos != -1 ) {
        QString path = exp.cap(1);
        QVariantMap hash = d->hashPathToRemoteUid.value(path);
        if ( d->hashPathToRemoteUid.contains(path) )
            d->contacts.append(hash);

        pos = exp.indexIn(inputText, pos+exp.matchedLength());
    }

    emit contactsChanged();

    // - Check if it's a number, then we accept the entry as it is.
    // - Otherwise we mark this entry as failed

    pos = d->plainText.indexOf(',');
    if ( pos != -1 ) {
        // Remove from trimmed text
        QString text = d->trimmedText;
        text.remove(',');

        ResolveState state = Resolved;

        if ( text.startsWith("+") || text.startsWith("00") ) {
            for(int i=0; i < text.size(); ++i) {
                const QChar c = text.at(i);

                if ( !c.isNumber() && c.toAscii() != '+' ) {
                    state = Unresolved;
                    break;
                }
            }
        } else
            state = Unresolved;

        // Create the image
        QString path = d->saveContactImage(text, state);
        QString insert = QString("<img src='%1'/> ").arg(path);

        if ( state == Resolved ) {
            QVariantMap hash;
            hash.insert("name", QString());
            hash.insert("number", text);

            d->hashPathToRemoteUid.insert(path, hash);
        }

        QTextCursor cursor = document.find(d->trimmedText);
        if ( cursor.isNull() )
            qDebug() << "Cursor is null";
        else {
            cursor.movePosition(QTextCursor::WordRight, QTextCursor::KeepAnchor);
            cursor.insertHtml(insert);

            d->numberOfContacts++;

            emit adjustTextEdit(document.toHtml(), d->numberOfContacts * 2);
        }

        d->contactName = QString();
    } else {
        d->contactName = d->plainText.trimmed();
    }

    emit contactNameChanged();
    emit isEmptyChanged();
}

void ContactTextEditFormatter::acceptSuggestedContact(const QString& name, const QString& phoneNumber)
{
    d->contactName = QString();

    ResolveState state = (phoneNumber.startsWith("+") || phoneNumber.startsWith("00")) ? Resolved : Unresolved;

    QString path = d->saveContactImage(name, state);
    QString insert = QString("<img src='%1'/> ").arg(path);

    if ( state == Resolved ) {
        QVariantMap hash;
        hash.insert("name", name);
        hash.insert("number", phoneNumber);

        d->hashPathToRemoteUid.insert(path, hash);
    }
    else {
        emit error( qtTrId("qtn_web2sms_no_international_number_title") );
    }

    QTextDocument document;
    document.setDefaultFont(d->defaultFont);
    document.setHtml(d->inputText);

    // If the text is empty, we append the information at the end
    if ( d->trimmedText.isEmpty() ) {
        QTextCursor cursor(&document);
        cursor.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
        cursor.insertHtml(insert);
    } else {
        QTextCursor cursor = document.find(d->trimmedText);
        if ( cursor.isNull() ) {
            qDebug() << "Cursor is null";
        }
        else {
            cursor.movePosition(QTextCursor::WordRight, QTextCursor::KeepAnchor);
            cursor.insertHtml(insert);
        }
    }

    d->numberOfContacts++;

    emit adjustTextEdit(document.toHtml(), d->numberOfContacts * 2);
    emit contactNameChanged();
    emit isEmptyChanged();
}

#include "contacttexteditformatter.moc"
