// Local includes
#include "providerupdateinstaller.h"
#include <pluginresolver.h>

// Global includes
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QUrl>
#include <QFile>
#include <QDesktopServices>
#include <QDir>
#include <QDebug>

class ProviderUpdateInstallerPrivate : public QObject
{
    Q_OBJECT

public:
    ProviderUpdateInstallerPrivate()
    : state(ProviderUpdateInstaller::Unknown)
    , progress(0.0)
    , network(NULL)
    {
    }

    ProviderUpdateInstaller* self;

    // Properties
    ProviderUpdateInstaller::State state;
    QString errorString;
    qreal progress;

    QNetworkAccessManager* network;

    // Functions
    void setState(ProviderUpdateInstaller::State state);
    void setErrorString(const QString& errorString);
    void setProgress(qreal progress);

public slots:
    void didFinishRequest();
    void downloadProgress(qint64 bytesReceived, qint64 bytesTotal);
};

void ProviderUpdateInstallerPrivate::setState(ProviderUpdateInstaller::State state)
{
    if ( this->state == state )
        return;

    this->state = state;
    emit self->stateChanged();
}

void ProviderUpdateInstallerPrivate::setErrorString(const QString &errorString)
{
    if ( this->errorString == errorString )
        return;

    this->errorString = errorString;
    emit self->errorStringChanged();
}

void ProviderUpdateInstallerPrivate::setProgress(qreal progress)
{
    if ( this->progress == progress )
        return;

    this->progress = progress;
    emit self->progressChanged();
}

void ProviderUpdateInstallerPrivate::didFinishRequest()
{
    // Get the sending reply back
    QNetworkReply* reply = qobject_cast<QNetworkReply*>( sender() );

    if ( reply->error() ) {
        setErrorString(reply->errorString());
        setState(ProviderUpdateInstaller::Error);
    } else {
        if ( reply->header(QNetworkRequest::ContentTypeHeader).toString() != "application/zip" ) {
            //% "Server replied with wrong content type"
            setErrorString( qtTrId("qtn_web2sms_providerupdateinstaller_wrong_content_type") );
            setState(ProviderUpdateInstaller::Error);
        } else {
            // Cd into the home directory
            QDir homeDir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
            if ( homeDir.cd(".web2sms") ) {
                // Unload the plugins
                PluginResolver::instance()->unload();

                // Try to open the file for writing
                QFile file(homeDir.absoluteFilePath("plugins.zip"));
                if ( file.open(QIODevice::WriteOnly) ) {
                    // Write the file contents
                    file.write( reply->readAll() );
                    file.close();

                    qDebug() << "Downloaded plugin update to" << file.fileName();

                    // Update states
                    setErrorString(QString());
                    setState(ProviderUpdateInstaller::Done);
                } else {
                    qDebug() << "Failed to open plugin file for writing";

                    //% "Failed to open plugin file for writing"
                    setErrorString(qtTrId("qtn_web2sms_providerupdateinstaller_cannot_open"));
                    setState(ProviderUpdateInstaller::Error);
                }

                // Kick the PluginResolver to reload the plugins
                PluginResolver::instance()->resolvePlugins();
            } else {
                qDebug() << "Failed to change into $HOME/.web2sms";

                //% "Failed to change into $HOME/.web2sms"
                setErrorString(qtTrId("qtn_web2sms_providerupdateinstaller_cannot_chdir"));
                setState(ProviderUpdateInstaller::Error);
            }
        }
    }

    // Cleanup memory
    reply->deleteLater();
}

void ProviderUpdateInstallerPrivate::downloadProgress(qint64 bytesReceived, qint64 bytesTotal)
{
    // Update the progress bar
    qreal progress = qreal(bytesReceived) / qreal(bytesTotal);
    setProgress(progress);
}

ProviderUpdateInstaller::ProviderUpdateInstaller(QObject *parent)
: QObject(parent)
, d(new ProviderUpdateInstallerPrivate)
{
    d->self = this;
}

ProviderUpdateInstaller::~ProviderUpdateInstaller()
{
    delete d;
}

ProviderUpdateInstaller::State ProviderUpdateInstaller::state() const
{
    return d->state;
}

QString ProviderUpdateInstaller::errorString() const
{
    return d->errorString;
}

qreal ProviderUpdateInstaller::progress() const
{
    return d->progress;
}

void ProviderUpdateInstaller::setNetworkAccessManager(QNetworkAccessManager *networkAccessManager)
{
    if ( d->network == networkAccessManager )
        return;

    d->network = networkAccessManager;
}

QNetworkAccessManager *ProviderUpdateInstaller::networkAccessManager() const
{
    return d->network;
}

void ProviderUpdateInstaller::installUpdate()
{
    // Set default values
    d->setState(Installing);
    d->setErrorString(QString());
    d->setProgress(0.0);

    /// TODO: Don't use a hardcoded url
    QNetworkRequest request( QUrl("http://web2sms.olausson.de/plugins.zip") );
    QNetworkReply* reply = d->network->get(request);
    connect(reply, SIGNAL(finished()), d, SLOT(didFinishRequest()));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), d, SLOT(downloadProgress(qint64,qint64)));
}

#include "providerupdateinstaller.moc"
